module APEX_ARINC653_TYPES =
   constant integer MAX_NAME_LENGTH = 30;
   constant integer MAX_NUMBER_OF_PARTITIONS = 20;
   constant integer MAX_NUMBER_OF_PROCESSES = 63;
   constant integer MAX_NUMBER_OF_SAMPLING_PORTS = 100;
   constant integer MAX_NUMBER_OF_QUEUING_PORTS = 100;
   constant integer MAX_NUMBER_OF_BUFFERS = 100;
   constant integer MAX_NUMBER_OF_BLACKBOARDS = 100;
   constant integer MAX_NUMBER_OF_SEMAPHORES = 100;
   constant integer MAX_SEMAPHORE_VALUE = 32767;
   constant integer MAX_NUMBER_OF_EVENTS = 63;
   type MessageSize_type = integer;
   constant MessageSize_type MAX_MESSAGE_SIZE = 1024;
   type BufferSize_type = integer;
   constant BufferSize_type MAX_BUFFER_MESSAGE_SIZE = 1024;
   constant BufferSize_type MAX_BUFFER_SIZE = 8*MAX_BUFFER_MESSAGE_SIZE;
   type QueuingPortSize_type = integer;
   constant QueuingPortSize_type MAX_QUEUING_PORT_SIZE = 8*MAX_MESSAGE_SIZE;
   type ProcessID_type = integer;
   type MessageArea_type = real;
   type Message_type = struct ( MessageArea_type Message_Area;
                                MessageSize_type Message_Size; );
   type Comm_ComponentID_type = integer;
   type Comm_ComponentName_type = string;
   type QueuingDiscipline_type = enum (FIFO, PRIORITY);
   type BlackboardStatus_type = struct ( boolean Empty_Indicator;
                                         MessageSize_type Message_Size;
                                         WaitingRange_type Waiting_Processes; );
   type APEX_Blackboard_type = struct ( Comm_ComponentName_type Blackboard_Name;
                                        Message_type Message;
                                        MessageSize_type Message_Size;
                                        boolean Empty_Indicator;
                                        WaitingRange_type Waiting_Processes;
                                        [MAX_NUMBER_OF_PROCESSES]boolean Waiting_Queue; );
   type BufferStatus_type = struct ( MessageRange_type Nb_Message;
                                     MessageRange_type Max_Message;
                                     MessageSize_type Message_Size;
                                     WaitingRange_type Waiting_Processes; );
   type APEX_Buffer_type = struct ( Comm_ComponentName_type Buffer_Name;
                                    [MAX_BUFFER_SIZE]Message_type Message_Range;
                                    MessageRange_type Nb_Message;
                                    MessageRange_type Max_Message;
                                    MessageSize_type Message_Size;
                                    WaitingRange_type WaitingOnSend_Processes;
                                    WaitingRange_type WaitingOnReceive_Processes;
                                    QueuingDiscipline_type Queuing_Discipline;
                                    QueueAddress_type WaitingOnSend_Queue;
                                    QueueAddress_type WaitingOnReceive_Queue; );
   type EventStateValue_type = enum (UP, DOWN);
   type EventStatus_type = struct ( EventStateValue_type Event_State;
                                    WaitingRange_type Waiting_Processes; );
   type APEX_Event_type = struct ( Comm_ComponentName_type Event_Name;
                                   EventStateValue_type Event_State;
                                   WaitingRange_type Waiting_Processes;
                                   [MAX_NUMBER_OF_PROCESSES]boolean Waiting_Queue; );
   type SemaphoreValue_type = integer;
   type SemaphoreStatus_type = struct ( SemaphoreValue_type Current_Value;
                                        SemaphoreValue_type Maximum_Value;
                                        WaitingRange_type Waiting_Processes; );
   type APEX_Semaphore_type = struct ( Comm_ComponentName_type Semaphore_Name;
                                       SemaphoreValue_type Current_Value;
                                       SemaphoreValue_type Maximum_Value;
                                       WaitingRange_type Waiting_Processes;
                                       QueuingDiscipline_type Queuing_Discipline;
                                       QueueAddress_type Waiting_Queue; );
   type ProcessName_type = string;
   type Priority_type = integer;
   type StackSize_type = integer;
   type LockLevel_type = integer;
   type SystemAddress_type = real;
   type ProcessState_type = enum (DORMANT, READY, RUNNING, WAITING);
   type Deadline_type = enum (SOFT, HARD);
   type SystemTime_type = real;
   type ProcessAttributes_type = struct ( ProcessName_type Name;
                                          SystemAddress_type Entry_Point;
                                          StackSize_type Stack_Size;
                                          Priority_type Base_Priority;
                                          SystemTime_type Period;
                                          SystemTime_type Time_Capacity;
                                          Deadline_type Deadline; );
   type ProcessStatus_type = struct ( ProcessAttributes_type Attributes;
                                      Priority_type Current_Priority;
                                      SystemTime_type Deadline_Time;
                                      ProcessState_type Process_State; );
   type ReturnCode_type = enum (NO_ERROR, NO_ACTION, NOT_AVAILABLE, INVALID_PARAM, INVALID_CONFIG, INVALID_MODE, TIMED_OUT);
   type OperatingMode_type = enum (IDLE, COLD_START, WARM_START, NORMAL);
   type PartitionID_type = integer;
   type PartitionStatus_type = struct ( PartitionID_type Identifier;
                                        SystemTime_type Period;
                                        SystemTime_type Duration;
                                        LockLevel_type LockLevel;
                                        OperatingMode_type OperatingMode; );
   type PortDirection_type = enum (SOURCE, DESTINATION);
   type SamplingPortSize_type = integer;
   type SamplingPortStatus_type = struct ( SamplingPortSize_type SamplingPort_Size;
                                           PortDirection_type Port_Direction;
                                           SystemTime_type Refresh_Period;
                                           boolean Validity; );
   type APEX_SamplingPort_type = struct ( Comm_ComponentName_type SamplingPort_Name;
                                          SamplingPortSize_type SamplingPort_Size;
                                          PortDirection_type Port_Direction;
                                          SystemTime_type Refresh_Period;
                                          boolean Validity;
                                          boolean Empty;
                                          SystemTime_type Message_Age;
                                          Message_type Message; );
   type QueuingPortStatus_type = struct ( QueuingPortSize_type QueuingPort_Size;
                                          PortDirection_type Port_Direction;
                                          MessageRange_type Nb_Message;
                                          WaitingRange_type Waiting_Processes; );
   type APEX_QueuingPort_type = struct ( Comm_ComponentName_type QueuingPort_Name;
                                         QueuingPortSize_type QueuingPort_Size;
                                         PortDirection_type Port_Direction;
                                         MessageSize_type Message_Size;
                                         MessageRange_type Nb_Message;
                                         [MAX_QUEUING_PORT_SIZE]Message_type Message_Range;
                                         QueuingDiscipline_type Queuing_Discipline;
                                         WaitingRange_type WaitingOnSend_Processes;
                                         WaitingRange_type WaitingOnReceive_Processes;
                                         QueueAddress_type WaitingOnSend_Queue;
                                         QueueAddress_type WaitingOnReceive_Queue; );
   constant ProcessID_type ERROR_HANDLER = 0;
   constant SystemTime_type MAX_TIMEOUT_VALUE = 100.0;
   constant SystemAddress_type MAX_ENTRY_POINT_ADDRESS = 99999.9;
   constant StackSize_type MAX_STACK_SIZE_VALUE = 100;
   constant SystemTime_type MAX_PERIOD_VALUE = 100.0;
   constant SystemTime_type MAX_TIME_CAPACITY_VALUE = 100.0;
   constant SystemTime_type APERIODIC = -1.0;
   constant SystemTime_type INFINITE_TIME_VALUE = -99.0;
   constant LockLevel_type MAX_LOCK_LEVEL = 16;
   constant Priority_type MAX_PRIORITY_VALUE = 63;
   constant SamplingPortSize_type MAX_SAMPLING_PORT_SIZE = MAX_MESSAGE_SIZE;
   constant BlackboardSize_type MAX_BLACKBOARD_MESSAGE_SIZE = 1024;
   constant BlackboardSize_type MAX_BLACKBOARD_SIZE = MAX_BLACKBOARD_MESSAGE_SIZE;
end %APEX_ARINC653_TYPES%; 
