module APEX_Blackboard_lib =

   process CREATE_BLACKBOARD =
        ( ? Comm_ComponentName_type blackboard_name;
            MessageSize_type message_size;
          ! Comm_ComponentID_type blackboard_ID;
            ReturnCode_type return_code;
        )
      (| (| (| (blackboard_ID,recorded) := BLACKBOARD_RECORD{}(blackboard_name when OK_perform,message_size when 
                                             OK_perform)
             | return_code := (#INVALID_CONFIG when full) default (#NO_ACTION when present) default (#INVALID_PARAM when
                              (not valid_param)) default (#INVALID_MODE when (op_mode=#NORMAL)) default #NO_ERROR
             | return_code ^= message_size ^= blackboard_name ^= full ^= op_mode
             |)
          | (| full := BLACKBOARD_CHECKCAPACITY{}()
             | (present,some_blackboardID) := BLACKBOARD_CHECKNAME{}(blackboard_name)
             | op_mode := PROCESS_CHECKOPERATINGMODE{}()
             | valid_param := message_size<=MAX_MESSAGE_SIZE
             | OK_perform := (when (not full)) when (not present) when (not (op_mode=#NORMAL)) when valid_param
             |)
          |)/ op_mode, valid_param, present, full, OK_perform |)
      where 
      boolean recorded, full, present, valid_param;
      Comm_ComponentID_type some_blackboardID;
      event OK_perform;
      OperatingMode_type op_mode;
      end
   ; 
   process READ_BLACKBOARD =
        { ProcessID_type process_ID; }
        ( ? Comm_ComponentID_type blackboard_ID;
            SystemTime_type timeout;
          ! MessageArea_type message;
            MessageSize_type length;
            ReturnCode_type return_code;
        )
      (| (| (present,blackboard_in) := BLACKBOARD_CHECKID{}(blackboard_ID)
          | (| outofrange := timeout>MAX_TIMEOUT_VALUE
             | available := timeout>0.0
             |)
          | (| (| (| empty := blackboard_in.Empty_Indicator when (not outofrange)
                   | err_handler := PROCESS_CHECKERRORHANDLER{}(process_ID when empty when available)
                   | enabled := PROCESS_CHECKPREEMPTION{}(when (not err_handler))
                   |)
                | enabled_1 := enabled
                | empty_2 := empty
                | err_handler_3 := err_handler
                |) where boolean enabled, empty, err_handler; end
             | (| empty := empty_2
                | enabled := enabled_1
                | (| PROCESS_SETSTATE{}(process_ID when enabled,#WAITING)
                   | INSERT_BLACKBOARDQUEUE{}(process_ID when enabled,blackboard_in when enabled)
                   | START_COUNTER{}(process_ID when enabled when (not (timeout=INFINITE_TIME_VALUE)),timeout when 
                                     enabled when (not (timeout=INFINITE_TIME_VALUE)))
                   | diagnostic := PROCESS_SCHEDULINGREQUEST{}(when enabled)
                   | mess := blackboard_in.Message when (not empty)
                   |)
                | mess_4 := mess
                |) where boolean enabled, empty;
                         Message_type mess; end
             |)
          | (| mess := mess_4
             | err_handler := err_handler_3
             | empty := empty_2
             | enabled := enabled_1
             | (| message := mess.Message_Area
                | length := mess.Message_Size
                | return_code := (#INVALID_PARAM when ((not present) or outofrange)) default (#NOT_AVAILABLE when empty 
                                 when (not available)) default (#INVALID_MODE when ((not enabled) default err_handler)) 
                                 default (#NO_ERROR when (not empty))
                | blackboard_ID ^= timeout
                |)
             |) where boolean enabled, empty, err_handler;
                      Message_type mess; end
          |)/ mess_4, err_handler_3, empty_2, enabled_1, available, outofrange, blackboard_in, present |)
      where 
      APEX_Blackboard_type blackboard_out, blackboard_in;
      event blocked;
      boolean diagnostic, present, outofrange, available, enabled_1, enabled, empty_2, empty, err_handler_3, err_handler
             ;
      Message_type mess_4, mess;
      end
   ; 
   process DISPLAY_BLACKBOARD =
        ( ? Comm_ComponentID_type blackboard_ID;
            MessageArea_type message;
            MessageSize_type length;
          ! ReturnCode_type return_code;
        )
      (| (| (present,blackboard_in) := BLACKBOARD_CHECKID{}(blackboard_ID)
          | (| (| (| toolong := (length when present)>blackboard_in.Message_Size
                   | mess.Message_Area := message when (not toolong)
                   | mess.Message_Size := length when (not toolong)
                   | blackboard_out := DISPLAYBOARD{}(blackboard_in when (not toolong),mess)
                   | release_all := (blackboard_in.Waiting_Processes>0) when (not toolong)
                   | process_range := REMOVALL_BLACKBOARDQUEUE{}(blackboard_out when release_all)
                   | STOPALL_COUNTER{}(process_range)
                   | PROCESS_SETALLSTATE{}(process_range,#READY)
                   | enabled := PROCESS_CHECKPREEMPTION{}(when (^process_range))
                   |)
                | toolong_1 := toolong
                |) where boolean toolong; end
             | diagnostic := PROCESS_SCHEDULINGREQUEST{}(when enabled)
             |)/ enabled
          | (| toolong := toolong_1
             | (| return_code := (#INVALID_PARAM when (toolong default (not present))) default #NO_ERROR
                | return_code ^= blackboard_ID ^= message ^= length
                |)
             |) where boolean toolong; end
          |)/ toolong_1, present, blackboard_in |)
      where 
      APEX_Blackboard_type blackboard_out, blackboard_in;
      boolean release_all, present, preemp_ON, diagnostic, enabled, toolong_1, toolong;
      [MAX_NUMBER_OF_PROCESSES]boolean process_range;
      Message_type mess;
      process DISPLAYBOARD =
           ( ? APEX_Blackboard_type blackboard_IN;
               Message_type message;
             ! APEX_Blackboard_type blackboard_OUT;
           )
         (| blackboard_OUT.Blackboard_Name := blackboard_IN.Blackboard_Name
          | blackboard_OUT.Message := message
          | blackboard_OUT.Message_Size := blackboard_IN.Message_Size
          | blackboard_OUT.Empty_Indicator := ((not blackboard_IN.Empty_Indicator) when blackboard_IN.Empty_Indicator) 
                                             default blackboard_IN.Empty_Indicator
          | blackboard_OUT.Waiting_Processes := blackboard_IN.Waiting_Processes-blackboard_IN.Waiting_Processes
          | blackboard_OUT.Waiting_Queue := blackboard_IN.Waiting_Queue
          | BLACKBOARD_UPDATE{}(blackboard_OUT)
          |)
      ; 
      end
   ; 
   process CLEAR_BLACKBOARD =
        ( ? Comm_ComponentID_type blackboard_ID;
          ! ReturnCode_type return_code;
        )
      (| (| (present,blackboard_in) := BLACKBOARD_CHECKID{}(blackboard_ID)
          | (| return_code := (#INVALID_PARAM when (not present)) default #NO_ERROR
             | return_code ^= blackboard_ID
             |)
          | blackboard_out := CLEARBOARD{}(blackboard_in)
          |)/ blackboard_in, present |)
      where 
      APEX_Blackboard_type blackboard_out, blackboard_in;
      process CLEARBOARD =
           ( ? APEX_Blackboard_type blackboard_IN;
             ! APEX_Blackboard_type blackboard_OUT;
           )
         (| blackboard_OUT.Blackboard_Name := blackboard_IN.Blackboard_Name
          | blackboard_OUT.Message := blackboard_IN.Message
          | blackboard_OUT.Empty_Indicator := ((not blackboard_IN.Empty_Indicator) when (not blackboard_IN.
                                             Empty_Indicator)) default blackboard_IN.Empty_Indicator
          | blackboard_OUT.Message_Size := blackboard_IN.Message_Size
          | blackboard_OUT.Waiting_Processes := blackboard_IN.Waiting_Processes
          | blackboard_OUT.Waiting_Queue := blackboard_IN.Waiting_Queue
          | BLACKBOARD_UPDATE{}(blackboard_OUT)
          |)
      ; 
      boolean present;
      end
   ; 
   process GET_BLACKBOARD_ID =
        ( ? Comm_ComponentName_type blackboard_name;
          ! Comm_ComponentID_type blackboard_ID;
            ReturnCode_type return_code;
        )
      (| (| (present,blackboard_ID) := BLACKBOARD_CHECKNAME{}(blackboard_name)
          | (| return_code := (#INVALID_CONFIG when (not present)) default #NO_ERROR
             | blackboard_name ^= return_code
             |)
          |)/ present |)
      where 
      boolean present;
      end
   ; 
   process GET_BLACKBOARD_STATUS =
        ( ? Comm_ComponentID_type blackboard_ID;
          ! BlackboardStatus_type blackboard_status;
            ReturnCode_type return_code;
        )
      (| (| (present,blackboard_in) := BLACKBOARD_CHECKID{}(blackboard_ID)
          | (| blackboard_status.Empty_Indicator := blackboard_in.Empty_Indicator
             | blackboard_status.Message_Size := blackboard_in.Message_Size
             | blackboard_status.Waiting_Processes := blackboard_in.Waiting_Processes
             |)
          | (| return_code := (#INVALID_PARAM when (not present)) default #NO_ERROR
             | return_code ^= blackboard_ID
             |)
          |)/ present, blackboard_in |)
      where 
      APEX_Blackboard_type blackboard_in;
      boolean present;
      end
   ; 
end %APEX_Blackboard_lib%; 
