module APEX_Buffer_lib =

   process CREATE_BUFFER =
        ( ? Comm_ComponentName_type buffer_name;
            BufferSize_type buffer_size;
            MessageSize_type message_size;
            QueuingDiscipline_type queuing_discipline;
          ! Comm_ComponentID_type buffer_ID;
            ReturnCode_type return_code;
        )
     (| (| (buffer_ID,recorded) := BUFFER_RECORD{}(buffer_name when OK_perform,buffer_size when OK_perform,
                                 message_size when OK_perform,queuing_discipline when OK_perform)
         | return_code := (#INVALID_CONFIG when full) default (#NO_ACTION when present) default (#INVALID_PARAM when
                          (not valid_param)) default (#INVALID_MODE when (op_mode=#NORMAL)) default #NO_ERROR
         | return_code ^= buffer_size ^= message_size ^= queuing_discipline ^= buffer_name ^= full ^= op_mode
         |)
      | (| full := BUFFER_CHECKCAPACITY{}()
         | (present,some_bufferID) := BUFFER_CHECKNAME{}(buffer_name)
         | op_mode := PROCESS_CHECKOPERATINGMODE{}()
         | valid_param := (buffer_size<=MAX_BUFFER_SIZE) and (message_size<=MAX_MESSAGE_SIZE) and ((
                          queuing_discipline=#FIFO) or (queuing_discipline=#PRIORITY))
         | OK_perform := (when (not full)) when (not present) when (not (op_mode=#NORMAL)) when valid_param
         |)
      |)
      where 
      Comm_ComponentID_type some_bufferID;
      boolean recorded, full, present, valid_param;
      event OK_perform;
      OperatingMode_type op_mode;
      end
   ; 
   process SEND_BUFFER =
        { ProcessID_type process_ID; }
        ( ? Comm_ComponentID_type buffer_ID;
            MessageArea_type message;
            MessageSize_type length;
            SystemTime_type timeout;
          ! ReturnCode_type return_code;
        )
        (| (| blocked := (buffer_ID ^- return_code) default (false when (^buffer_ID))
            | zblocked := blocked$1 init false
            | on_waiting := zblocked when (^buffer_ID)
            |)
         | return_code_1 := SEND{process_ID}(buffer_ID when (not on_waiting), message when (not on_waiting), 
                                             length when (not on_waiting), timeout when (not on_waiting))
         | (| buffer_ID ^= message ^= length ^= timeout
            | return_code := return_code_1 default (#NO_ERROR when on_waiting)
            |)
         |)
      where 
        ReturnCode_type return_code_1;
        boolean blocked, zblocked, on_waiting;
      process SEND =
           { ProcessID_type process_ID; }
           ( ? Comm_ComponentID_type buffer_ID;
               MessageArea_type message;
               MessageSize_type length;
               SystemTime_type timeout;
             ! ReturnCode_type return_code;
           )
            (| (present,buffer_in) := BUFFER_CHECKID{}(buffer_ID)
             | (| outofrange := timeout>MAX_TIMEOUT_VALUE
                | available := timeout>0.0
                |)
             | (| (| enabled := enabled_3
                   | (| PROCESS_SETSTATE{}(process_ID when enabled,#WAITING)
                      | INSERT_BUFFERSENDQUEUE{}(process_ID when enabled,mess when enabled,buffer_in when enabled)
                      | START_COUNTER{}(process_ID when enabled when (not (timeout=INFINITE_TIME_VALUE)),timeout when 
                                        enabled when (not (timeout=INFINITE_TIME_VALUE)))
                      |)
                   |) where boolean enabled; end
                | (| some_processID := REMOVE_BUFFERRECEIVEQUEUE{}(buffer_in when (not nowaitingprocess))
                   | PROCESS_COPYMESSAGE(mess when (not nowaitingprocess),some_processID when (not nowaitingprocess))
                   | STOP_COUNTER{}(some_processID)
                   | PROCESS_SETSTATE{}(some_processID,#READY)
                   |)
                | (| (| full := (buffer_in.Nb_Message=buffer_in.Max_Message) when (not outofrange)
                      | toolong := (length when (not full))>(buffer_in.Message_Size when (not full))
                      | nowaitingprocess := (buffer_in.WaitingOnReceive_Processes=0) when (not toolong)
                      | mess.Message_Area := message when (^buffer_in)
                      | mess.Message_Size := length when (^buffer_in)
                      | buffer_out := SET_MESSAGE{}(buffer_in when nowaitingprocess,mess when nowaitingprocess)
                      | err_handler := PROCESS_CHECKERRORHANDLER{}(process_ID when full when available)
                      | enabled := PROCESS_CHECKPREEMPTION{}(when (not err_handler))
                      |)
                   | err_handler_1 := err_handler
                   | full_2 := full
                   | enabled_3 := enabled
                   | toolong_4 := toolong
                   |) where boolean err_handler, full, enabled, toolong; end
                | (| enabled := enabled_3
                   | diagnostic := PROCESS_SCHEDULINGREQUEST{}(some_processID ^+ (when enabled))
                   |) where boolean enabled; end
                |)/ nowaitingprocess, mess, some_processID
             | (| toolong := toolong_4
                | enabled := enabled_3
                | full := full_2
                | err_handler := err_handler_1
                | (| return_code := (#INVALID_PARAM when (toolong default ((not present) or outofrange))) default (#
                                    NOT_AVAILABLE when full when (not available)) default (#INVALID_MODE when ((not 
                                    enabled) default err_handler)) default (#NO_ERROR when (not full))
                   | buffer_ID ^= timeout ^= message ^= length
                   |)
                |) where boolean err_handler, full, enabled, toolong; end
             |)
         where 
         event blocked, ready_OK, OK_perform, timeout_OK;
         ProcessID_type some_processID;
         boolean diagnostic, present, outofrange, available, err_handler_2, err_handler, full_3, full, enabled_4, 
                enabled, nowaitingprocess, toolong_5, toolong, err_handler_1, full_2, enabled_3, toolong_4;
         APEX_Buffer_type buffer_out, buffer_in;
         process SET_MESSAGE =
              ( ? APEX_Buffer_type buffer_IN;
                  Message_type message_IN;
                ! APEX_Buffer_type buffer_OUT;
              )
           (| buffer_OUT.Buffer_Name := buffer_IN.Buffer_Name
            | aux := ([{i to (MAX_BUFFER_SIZE-1)}:buffer_IN.Message_Range[i],[buffer_IN.Nb_Message]:message_IN] when (buffer_IN.Nb_Message<buffer_IN.Max_Message))
            | buffer_OUT.Message_Range := aux default buffer_IN.Message_Range
            | buffer_OUT.Nb_Message := ((buffer_IN.Nb_Message+1) when (buffer_IN.Nb_Message<buffer_IN.Max_Message)) default buffer_IN.Nb_Message
            | buffer_OUT.Max_Message := buffer_IN.Max_Message
            | buffer_OUT.Message_Size := buffer_IN.Message_Size
            | buffer_OUT.WaitingOnSend_Processes := buffer_IN.WaitingOnSend_Processes
            | buffer_OUT.WaitingOnReceive_Processes := buffer_IN.WaitingOnReceive_Processes
            | buffer_OUT.Queuing_Discipline := buffer_IN.Queuing_Discipline
            | buffer_OUT.WaitingOnSend_Queue := buffer_IN.WaitingOnSend_Queue
            | buffer_OUT.WaitingOnReceive_Queue := buffer_IN.WaitingOnReceive_Queue
            | BUFFER_UPDATE{}(buffer_out)
            |)
           where 
             [MAX_BUFFER_SIZE]Message_type aux;
           end; 
         Message_type mess;
         end; 
      end; 
   process RECEIVE_BUFFER =
        { ProcessID_type process_ID; }
        ( ? Comm_ComponentID_type buffer_ID;
            SystemTime_type timeout;
          ! MessageArea_type message;
            MessageSize_type length;
            ReturnCode_type return_code;
        )
      (| (| blocked := (false when (^message_2)) default (buffer_ID ^- message_1) default (false when (^buffer_ID))
             | zblocked := blocked$1 init false
             | retrieve := zblocked when (^buffer_ID)
             |)
          | (message_1,length_1,return_code_1) := RECEIVE{process_ID}(buffer_ID when (not retrieve),timeout when (not 
                                             retrieve))
          | message_2 := PROCESS_RETRIEVEMESSAGE{}(process_ID when retrieve)
          | (| message := message_2.Message_Area default message_1
             | length := message_2.Message_Size default length_1
             | buffer_ID ^= timeout
             | return_code := return_code_1 default (#NO_ERROR when (^message_2))
             |)
          |)
      where 
      boolean blocked, zblocked, retrieve;
      Message_type message_2;
      MessageArea_type message_1;
      MessageSize_type length_1;
      ReturnCode_type return_code_1;
      process RECEIVE =
           { ProcessID_type process_ID; }
           ( ? Comm_ComponentID_type buffer_ID;
               SystemTime_type timeout;
             ! MessageArea_type message;
               MessageSize_type length;
               ReturnCode_type return_code;
           )
          (| (present,buffer_in) := BUFFER_CHECKID{}(buffer_ID)
             | (| enabled := enabled_4
                | empty := empty_3
                | err_handler := err_handler_2
                | mess := mess_1
                | (| message := mess.Message_Area
                   | length := mess.Message_Size
                   | return_code := (#INVALID_PARAM when ((not present) or outofrange)) default (#NOT_AVAILABLE when 
                                    empty when (not available)) default (#INVALID_MODE when ((not enabled) default 
                                    err_handler)) default (#NO_ERROR when (not empty))
                   | buffer_ID ^= timeout
                   |)
                |) where Message_type mess;
                         boolean err_handler, empty, enabled; end
             | (| (| enabled := enabled_4
                   | (| PROCESS_SETSTATE{}(process_ID when enabled,#WAITING)
                      | INSERT_BUFFERRECEIVEQUEUE{}(process_ID when enabled,buffer_in when enabled)
                      | START_COUNTER{}(process_ID when enabled when (not (timeout=INFINITE_TIME_VALUE)),timeout when 
                                        enabled when (not (timeout=INFINITE_TIME_VALUE)))
                      |)
                   |) where boolean enabled; end
                | (| (some_processID,some_message) := REMOVE_BUFFERSENDQUEUE{}(buffer_in when (not nowaitingprocess))
                   | STOP_COUNTER{}(some_processID)
                   | PROCESS_SETSTATE{}(some_processID,#READY)
                   |)
                | (| (| empty := (buffer_in.Nb_Message=0) when (not outofrange)
                      | nowaitingprocess := (buffer_in.WaitingOnSend_Processes=0) when (not empty)
                      | (buffer_out,mess) := GET_MESSAGE{}(buffer_in when nowaitingprocess,var some_message)
                      | err_handler := PROCESS_CHECKERRORHANDLER{}(process_ID when empty when available)
                      | enabled := PROCESS_CHECKPREEMPTION{}(when (not err_handler))
                      |)
                   | mess_1 := mess
                   | err_handler_2 := err_handler
                   | empty_3 := empty
                   | enabled_4 := enabled
                   |) where Message_type mess;
                            boolean err_handler, empty, enabled; end
                | (| enabled := enabled_4
                   | diagnostic := PROCESS_SCHEDULINGREQUEST{}(some_processID ^+ (when enabled))
                   |) where boolean enabled; end
                |)/ nowaitingprocess, some_message, some_processID
             | (| outofrange := timeout>MAX_TIMEOUT_VALUE
                | available := timeout>0.0
                |)
             |)
         where 
         boolean nowaitingprocess, present, empty, preemp_OK, err_handler, preemp_ON_1, preemp_ON, available, 
                err_handler_1, empty_3, preemp_ON_4, outofrange, diagnostic, err_handler_2, enabled_4, enabled, 
                err_handler_3, empty_4, enabled_5;
         event OK_perform, notavailable, blocked, ready_OK, timeout_OK;
         APEX_Buffer_type buffer_out, buffer_in;
         ProcessID_type some_processID;
         process GET_MESSAGE =
              ( ? APEX_Buffer_type buffer_IN;
                  Message_type message_IN;
                ! APEX_Buffer_type buffer_OUT;
                  Message_type message_OUT;
              )
           (| (| buffer_OUT.Buffer_Name := buffer_IN.Buffer_Name
               | aux := ([{i to (MAX_BUFFER_SIZE-2)}:buffer_IN.Message_Range[i+1],[MAX_BUFFER_SIZE-1]:message_IN] when (^message_IN) when (buffer_IN.Nb_Message>0)) 
               | buffer_OUT.Message_Range := aux default buffer_IN.Message_Range
               | buffer_OUT.Nb_Message := ((buffer_IN.Nb_Message-1) when (buffer_IN.Nb_Message>0)) default buffer_IN.Nb_Message
               | buffer_OUT.Max_Message := buffer_IN.Max_Message
               | buffer_OUT.Message_Size := buffer_IN.Message_Size
               | buffer_OUT.WaitingOnSend_Processes := buffer_IN.WaitingOnSend_Processes
               | buffer_OUT.WaitingOnReceive_Processes := buffer_IN.WaitingOnReceive_Processes
               | buffer_OUT.Queuing_Discipline := buffer_IN.Queuing_Discipline
               | buffer_OUT.WaitingOnSend_Queue := buffer_IN.WaitingOnSend_Queue
               | buffer_OUT.WaitingOnReceive_Queue := buffer_IN.WaitingOnReceive_Queue
               | BUFFER_UPDATE{}(buffer_OUT)
               |)
            | (| mess := buffer_OUT.Message_Range[(MAX_BUFFER_SIZE-buffer_OUT.Nb_Message)-1]
               | message_OUT := (var mess) when (buffer_IN.Nb_Message>0)
               |)
            |)
           where
             [MAX_BUFFER_SIZE]Message_type aux;
             Message_type mess;
           end; 
         Message_type some_message, mess_1, mess;
         end; 
      end; 
   process GET_BUFFER_ID =
        ( ? Comm_ComponentName_type buffer_name;
          ! Comm_ComponentID_type buffer_ID;
            ReturnCode_type return_code;
        )
     (| (| return_code := (#INVALID_CONFIG when (not present)) default #NO_ERROR
         | buffer_name ^= return_code
         |)
      | (present,buffer_ID) := BUFFER_CHECKNAME{}(buffer_name)
      |)
     where 
       boolean present;
     end; 
   process GET_BUFFER_STATUS =
        ( ? Comm_ComponentID_type buffer_ID;
          ! BufferStatus_type buffer_status;
            ReturnCode_type return_code;
        )
     (| (| return_code := (#INVALID_PARAM when (not present)) default #NO_ERROR
         | return_code ^= buffer_ID
         |)
      | (present,buffer_in) := BUFFER_CHECKID{}(buffer_ID)
      | (| buffer_status.Nb_Message := buffer_in.Nb_Message
         | buffer_status.Max_Message := buffer_in.Max_Message
         | buffer_status.Message_Size := buffer_in.Message_Size
         | buffer_status.Waiting_Processes := buffer_in.WaitingOnSend_Processes+buffer_in.WaitingOnReceive_Processes
         |)
      |)
     where 
       APEX_Buffer_type buffer_in;
       boolean present;
     end; 
end %APEX_Buffer_lib%; 
