module APEX_Event_lib =

   process CREATE_EVENT =
        ( ? Comm_ComponentName_type event_name;
          ! Comm_ComponentID_type event_ID;
            ReturnCode_type return_code;
        )
      (| (| (| full := EVENT_CHECKCAPACITY{}()
             | (present,some_eventID) := EVENT_CHECKNAME{}(event_name)
             | op_mode := PROCESS_CHECKOPERATINGMODE{}()
             | OK_perform := (when (not full)) when (not present) when (not (op_mode=#NORMAL))
             |)
          | (| (event_ID,recorded) := EVENT_RECORD{}(event_name when OK_perform)
             | return_code := (#INVALID_CONFIG when full) default (#NO_ACTION when present) default (#INVALID_MODE when 
                              (op_mode=#NORMAL)) default #NO_ERROR
             | return_code ^= event_name ^= full ^= op_mode
             |)
          |)/ OK_perform, op_mode, present, full |)
      where 
      boolean recorded, full, present;
      Comm_ComponentID_type some_eventID;
      OperatingMode_type op_mode;
      event OK_perform;
      end
   ; 
   process WAIT_EVENT =
        { ProcessID_type process_ID; }
        ( ? Comm_ComponentID_type event_ID;
            SystemTime_type timeout;
          ! ReturnCode_type return_code;
        )
      (| (| (| blocked := (event_ID ^- return_code) default (false when (^event_ID))
             | zblocked := blocked$1 init false
             | on_waiting := zblocked when (^event_ID)
             |)
          | return_code_1 := WAIT{process_ID}(event_ID when (not on_waiting),timeout when (not on_waiting))
          | (| event_ID ^= timeout
             | return_code := return_code_1 default (#NO_ERROR when on_waiting)
             |)
          |)/ return_code_1, on_waiting |)
      where 
      boolean blocked, zblocked, on_waiting;
      process WAIT =
           { ProcessID_type process_ID; }
           ( ? Comm_ComponentID_type event_ID;
               SystemTime_type timeout;
             ! ReturnCode_type return_code;
           )
         (| (| (present,event_in) := EVENT_CHECKID{}(event_ID)
             | (| outofrange := timeout>MAX_TIMEOUT_VALUE
                | available := timeout>0.0
                |)
             | (| (| (| up := (event_in.Event_State=#UP) when (not outofrange)
                      | err_handler := PROCESS_CHECKERRORHANDLER{}(process_ID when (not up) when available)
                      | enabled := PROCESS_CHECKPREEMPTION{}(when (not err_handler))
                      |)
                   | err_handler_1 := err_handler
                   | up_2 := up
                   | enabled_3 := enabled
                   |) where boolean err_handler, up, enabled; end
                | (| enabled := enabled_3
                   | up := up_2
                   | (| PROCESS_SETSTATE{}(process_ID when enabled,#WAITING)
                      | INSERT_EVENTQUEUE{}(process_ID when enabled,event_in when enabled)
                      | START_COUNTER{}(process_ID when enabled when (not (timeout=INFINITE_TIME_VALUE)),timeout when 
                                        enabled when (not (timeout=INFINITE_TIME_VALUE)))
                      | diagnostic := PROCESS_SCHEDULINGREQUEST{}(when enabled)
                      |)
                   |) where boolean up, enabled; end/ diagnostic
                |)
             | (| enabled := enabled_3
                | up := up_2
                | err_handler := err_handler_1
                | (| return_code := (#INVALID_PARAM when ((not present) or outofrange)) default (#NOT_AVAILABLE when (
                                    not up) when (not available)) default (#INVALID_MODE when ((not enabled) default 
                                    err_handler)) default (#NO_ERROR when up)
                   | event_ID ^= timeout
                   |)
                |) where boolean err_handler, up, enabled; end
             |)/ enabled_3, up_2, err_handler_1, available, outofrange, event_in, present |)
         where 
         APEX_Event_type event_out, event_in;
         boolean diagnostic, present, outofrange, available, err_handler_1, err_handler, up_2, up, enabled_3, enabled;
         end
      ; 
      ReturnCode_type return_code_1;
      end
   ; 
   process SET_EVENT =
        ( ? Comm_ComponentID_type event_ID;
          ! ReturnCode_type return_code;
        )
      (| (| (present,event_in) := EVENT_CHECKID{}(event_ID)
          | (| (| event_out := SETEVT{}(event_in)
                | release_all := event_in.Waiting_Processes>0
                | process_range := REMOVALL_EVENTQUEUE{}(event_out when release_all)
                | STOPALL_COUNTER{}(process_range)
                | PROCESS_SETALLSTATE{}(process_range,#READY)
                | enabled := PROCESS_CHECKPREEMPTION{}(when (^process_range))
                |)
             | diagnostic := PROCESS_SCHEDULINGREQUEST{}(when enabled)
             |)/ enabled
          | (| return_code := (#INVALID_PARAM when (not present)) default #NO_ERROR
             | return_code ^= event_ID
             |)
          |)/ event_in, present |)
      where 
      boolean preemp_OK, release_all, present, diagnostic, enabled;
      APEX_Event_type event_out, event_in;
      [MAX_NUMBER_OF_PROCESSES]boolean process_range;
      process SETEVT =
           ( ? APEX_Event_type event_IN;
             ! APEX_Event_type event_OUT;
           )
         (| event_OUT.Event_Name := event_IN.Event_Name
          | event_OUT.Event_State := (#UP when (event_IN.Event_State=#DOWN)) default event_IN.Event_State
          | event_OUT.Waiting_Processes := event_IN.Waiting_Processes-event_IN.Waiting_Processes
          | event_OUT.Waiting_Queue := event_IN.Waiting_Queue
          | EVENT_UPDATE{}(event_OUT)
          |)
      ; 
      end
   ; 
   process RESET_EVENT =
        ( ? Comm_ComponentID_type event_ID;
          ! ReturnCode_type return_code;
        )
      (| (| (present,event_in) := EVENT_CHECKID{}(event_ID)
          | (| return_code := (#INVALID_PARAM when (not present)) default #NO_ERROR
             | return_code ^= event_ID
             |)
          | event_out := RESETEVT{}(event_in when present)
          |)/ event_in, present |)
      where 
      APEX_Event_type event_out, event_in;
      process RESETEVT =
           ( ? APEX_Event_type event_IN;
             ! APEX_Event_type event_OUT;
           )
         (| event_OUT.Event_Name := event_IN.Event_Name
          | event_OUT.Event_State := (#DOWN when (event_IN.Event_State=#UP)) default event_IN.Event_State
          | event_OUT.Waiting_Processes := event_IN.Waiting_Processes
          | event_OUT.Waiting_Queue := event_IN.Waiting_Queue
          | EVENT_UPDATE{}(event_OUT)
          |)
      ; 
      boolean present;
      end
   ; 
   process GET_EVENT_ID =
        ( ? Comm_ComponentName_type event_name;
          ! Comm_ComponentID_type event_ID;
            ReturnCode_type return_code;
        )
      (| (| (present,event_ID) := EVENT_CHECKNAME{}(event_name)
          | (| return_code := (#INVALID_CONFIG when (not present)) default #NO_ERROR
             | event_name ^= return_code
             |)
          |)/ present |)
      where 
      boolean present;
      end
   ; 
   process GET_EVENT_STATUS =
        ( ? Comm_ComponentID_type event_ID;
          ! EventStatus_type event_status;
            ReturnCode_type return_code;
        )
      (| (| (present,event_in) := EVENT_CHECKID{}(event_ID)
          | (| event_status.Event_State := event_in.Event_State
             | event_status.Waiting_Processes := event_in.Waiting_Processes
             |)
          | (| return_code := (#INVALID_PARAM when (not present)) default #NO_ERROR
             | return_code ^= event_ID
             |)
          |)/ present, event_in |)
      where 
      APEX_Event_type event_in;
      boolean present;
      end
   ; 
end %APEX_Event_lib%; 
