module APEX_Partition_Manager_lib =
   use TYPES_AND_CONSTANTS_lib; 
   use APEX_Partition_Descriptor_lib; 
   process GET_PARTITION_STATUS =
        ( ! PartitionStatus_type partition_status;
            ReturnCode_type return_code;
        )
      (| (partition_ID,partition_status) := PARTITION_GETCURRENTID{}()
       | return_code := #NO_ERROR
       |)
      where 
      PartitionID_type partition_ID;
      end
   ; 
   process SET_PARTITION_MODE =
        ( ? OperatingMode_type operating_mode;
          ! ReturnCode_type return_code;
        )
      (| (| (| valid_opmode := (operating_mode=#IDLE) or (operating_mode=#COLD_START) or (operating_mode=#WARM_START) or
                               (operating_mode=#NORMAL)
             | (partition_ID,status) := PARTITION_GETCURRENTID{}()
             | no_action := (operating_mode=#NORMAL) and (status.OperatingMode=#NORMAL)
             | status ^= no_action
             | OK_req := valid_opmode and (not no_action)
             |)
          | (| PARTITION_SETMODE{}(partition_ID,operating_mode)
             | PARTITION_SHUTDOWN{}(partition_ID when (operating_mode=#IDLE))
             | PARTITION_INHIBITESCHEDULING{}(partition_ID when ((operating_mode=#WARM_START) or (operating_mode=#
                                             COLD_START)))
             | PARTITION_INITIALIZE{}(partition_ID when ((operating_mode=#WARM_START) or (operating_mode=#COLD_START)))
             | PARTITION_CALCULATEDEADLINETIME{}(partition_ID when (operating_mode=#NORMAL))
             | diagnostic := PARTITION_SCHEDULINGREQUEST{}(partition_ID when (operating_mode=#NORMAL))
             |)
          | (| return_code := (#INVALID_PARAM when (not valid_opmode)) default (#NO_ACTION when no_action) default #
                              NO_ERROR
             | return_code ^= operating_mode
             |)
          |)/ valid_opmode, no_action, partition_ID, OK_req |)
      where 
      PartitionStatus_type status;
      process PARTITION_GETCURRENTID =
           ( ! PartitionID_type partition_ID;
               PartitionStatus_type status;
           )
         pragmas 
            C_Code " "
         end pragmas
      
      ; 
      process PARTITION_SETMODE =
           ( ? PartitionID_type partition_ID;
               OperatingMode_type operating_mode;
           )
         pragmas 
            C_Code " "
         end pragmas
      
      ; 
      process PARTITION_SHUTDOWN =
           ( ? PartitionID_type partition_ID;
           )
         pragmas 
            C_Code " "
         end pragmas
      
      ; 
      process PARTITION_INHIBITESCHEDULING =
           ( ? PartitionID_type partition_ID;
           )
         pragmas 
            C_Code " "
         end pragmas
      
      ; 
      process PARTITION_INITIALIZE =
           ( ? PartitionID_type partition_ID;
           )
         pragmas 
            C_Code " "
         end pragmas
      
      ; 
      process PARTITION_CALCULATEDEADLINETIME =
           ( ? PartitionID_type partition_ID;
           )
         pragmas 
            C_Code " "
         end pragmas
      
      ; 
      process PARTITION_SCHEDULINGREQUEST =
           ( ? PartitionID_type pid;
             ! boolean diagnostic;
           )
         pragmas 
            C_Code " "
         end pragmas
      
      ; 
      boolean OK_req, no_action, valid_opmode, diagnostic;
      PartitionID_type partition_ID;
      end
   ; 
end %APEX_Partition_Manager_lib%; 
