module APEX_Process_Manager_lib =

   process GET_PROCESS_ID =
        ( ? ProcessName_type process_name;
          ! ProcessID_type process_ID;
            ReturnCode_type return_code;
        )
      (| (| (| return_code := (#INVALID_CONFIG when (not present)) default #NO_ERROR
             | return_code ^= process_name
             |)
          | (present,process_ID) := PROCESS_CHECKNAME{}(process_name)
          |)/ present |)
      where 
      boolean present;
      end
   ; 
   process GET_PROCESS_STATUS =
        ( ? ProcessID_type process_ID;
          ! ProcessStatus_type process_status;
            ReturnCode_type return_code;
        )
      (| (| (present,process_status) := PROCESS_CHECKID{}(process_ID)
          | (| return_code := (#INVALID_PARAM when (not present)) default #NO_ERROR
             | return_code ^= process_ID
             |)
          |)/ present |)
      where 
      boolean present;
      end
   ; 
   process SET_PRIORITY =
        ( ? ProcessID_type process_ID;
            Priority_type priority;
          ! ReturnCode_type return_code;
        )
      (| (| (| (present,status) := PROCESS_CHECKID{}(process_ID)
             | outofrange := priority>MAX_PRIORITY_VALUE
             | state := PROCESS_GETSTATE{}(process_ID)
             | OK_req := present and (not outofrange) and (not (state=#DORMANT))
             |)
          | (| return_code := (#INVALID_PARAM when ((not present) or outofrange)) default (#INVALID_MODE when (state=#
                              DORMANT)) default #NO_ERROR
             | return_code ^= process_ID ^= priority
             |)
          | (| PROCESS_SETPRIORITY{}(process_ID when OK_req,priority when OK_req)
             | enabled := PROCESS_CHECKPREEMPTION{}(when OK_req)
             | diagnostic := PROCESS_SCHEDULINGREQUEST{}(when enabled)
             |)
          |)/ present, state, outofrange, OK_req |)
      where 
      ProcessStatus_type status_OUT, status, status_IN;
      ProcessState_type state;
      boolean outofrange, present, OK_req, diagnostic, enabled;
      end
   ; 
   process START =
        ( ? ProcessID_type process_ID;
          ! ReturnCode_type return_code;
        )
      (| (| (| (present,status) := PROCESS_CHECKID{}(process_ID)
             | state := PROCESS_GETSTATE{}(process_ID when present)
             | OK_req := when (state=#DORMANT)
             |)
          | (| return_code := (#INVALID_PARAM when (not present)) default (#NO_ACTION when (not (state=#DORMANT))) 
                              default #NO_ERROR
             | return_code ^= process_ID
             |)
          | (| (| priority := PROCESS_GETBASICPRIORITY{}(process_ID when OK_req)
                | PROCESS_SETSTATE{}(process_ID when OK_req,#READY)
                | PROCESS_SETPRIORITY{}(process_ID when OK_req,priority)
                | PROCESS_RESETCONTEXT{}(process_ID when OK_req)
                | PROCESS_RESETSTACK{}(process_ID when OK_req)
                | op_mode := PROCESS_CHECKOPERATINGMODE{}()
                | (cur_time,ret) := GET_TIME{}()
                | normal_mode := (when OK_req) when (op_mode=#NORMAL)
                | time_capacity := PROCESS_GETTIMECAPACITY{}(process_ID when normal_mode)
                | PROCESS_SETDEADLINETIME{}(process_ID when normal_mode,time_capacity+cur_time)
                | PROCESS_RESETERRORSTATUS{}(when normal_mode)
                | ret ^= when normal_mode
                | op_mode ^= OK_req
                | enabled := PROCESS_CHECKPREEMPTION{}(when normal_mode)
                |)
             | diagnostic := PROCESS_SCHEDULINGREQUEST{}(when enabled)
             |)/ enabled
          |)/ state, present, OK_req |)
      where 
      ProcessStatus_type status_OUT, status, status_IN;
      SystemTime_type cur_time, time_capacity;
      boolean present, OK_req, diagnostic, enabled;
      OperatingMode_type op_mode;
      Priority_type priority;
      ReturnCode_type ret;
      event normal_mode;
      ProcessState_type state;
      end
   ; 
   process LOCK_PREEMPTION =
        ( ? LockLevel_type lock_level;
          ! ReturnCode_type return_code;
        )
      (| error := lock_level>=MAX_LOCK_LEVEL
       | PROCESS_LOCKLEVELINC{}(when (not error))
       | return_code := (#INVALID_CONFIG when error) default #NO_ERROR
       | return_code ^= lock_level
       |)
      where 
      boolean error;
      LockLevel_type lock_level_OUT;
      end
   ; 
   process UNLOCK_PREEMPTION =
        ( ? LockLevel_type lock_level;
          ! ReturnCode_type return_code;
        )
      (| unlocked := lock_level<=0
       | reschedule := PROCESS_LOCKLEVELDEC{}(when (not unlocked))
       | enabled := PROCESS_CHECKPREEMPTION{}(when reschedule)
       | diagnostic := PROCESS_SCHEDULINGREQUEST{}(when enabled)
       | return_code := (#NO_ACTION when unlocked) default #NO_ERROR
       | return_code ^= lock_level
       |)
      where 
      boolean diagnostic, reschedule, unlocked, enabled;
      process PROCESS_LOCKLEVELDEC =
           ( ? event dec_req;
             ! boolean reschedule;
           )
           spec (| (| dec_req --> reschedule |)
                 | (| dec_req ^= reschedule |)
                 |)
         pragmas 
            C_Code "GLOBAL_PROCESS_MANAGER->ProcessDecrementLockLevel(&o1)"
         end pragmas
      
      ; 
      end
   ; 
   process CREATE_PROCESS =
        ( ? ProcessAttributes_type attributes;
          ! ProcessID_type process_ID;
            ReturnCode_type return_code;
        )
      (| (| (| (present,process_ID) := PROCESS_CHECKNAME{}(attributes.Name)
             | created := PROCESS_CHECKCREATED{}(attributes.Name when present)
             | entrypt_OK := (attributes.Entry_Point<=MAX_ENTRY_POINT_ADDRESS) when present
             | stack_OK := (attributes.Stack_Size<=MAX_STACK_SIZE_VALUE) when present
             | baseprior_OK := (attributes.Base_Priority<=MAX_PRIORITY_VALUE) when present
             | period_OK := (attributes.Period<=MAX_PERIOD_VALUE) when present
             | timecap_OK := (attributes.Time_Capacity<=MAX_TIME_CAPACITY_VALUE) when present
             | op_mode := PROCESS_CHECKOPERATINGMODE{}()
             | OK_req := (true when (entrypt_OK and stack_OK and baseprior_OK and period_OK and timecap_OK and (not 
                         created) and (not (op_mode=#NORMAL)))) default false
             | OK_req ^= when present
             |)
          | (| return_code := (#INVALID_CONFIG when ((not (entrypt_OK and stack_OK and baseprior_OK and period_OK and 
                              timecap_OK)) default (not present))) default (#INVALID_MODE when (op_mode=#NORMAL)) 
                              default (#NO_ACTION when created) default #NO_ERROR
             | return_code ^= attributes ^= present
             |)
          | (| PROCESS_SETATTRIBUTES{}(process_ID when OK_req,var attributes)
             | PROCESS_SETSTATE{}(process_ID when OK_req,#DORMANT)
             | PROCESS_RESETCONTEXT{}(process_ID when OK_req)
             | PROCESS_RESETSTACK{}(process_ID when OK_req)
             | PROCESS_SETCREATED{}(process_ID when OK_req)
             |)
          |)/ present, op_mode, timecap_OK, period_OK, baseprior_OK, stack_OK, created, entrypt_OK, OK_req |)
      where 
      boolean OK_req, entrypt_OK, created, stack_OK, baseprior_OK, period_OK, timecap_OK, present;
      OperatingMode_type op_mode;
      end
   ; 
   process SUSPEND_SELF =
        ( ? SystemTime_type timeout;
          ! ReturnCode_type return_code;
        )
      (| (| (| (process_ID,status) := PROCESS_GETACTIVE{}(^timeout)
             | aperiodic := PROCESS_CHECKAPERIODIC{}(process_ID)
             | enabled := PROCESS_CHECKPREEMPTION{}(^process_ID)
             | err_handler := PROCESS_CHECKERRORHANDLER{}(process_ID)
             | outofrange := (timeout>MAX_TIMEOUT_VALUE) when (^process_ID)
             | nulltimeout := (timeout=0.0) when (^process_ID)
             | OK_req := (not outofrange) and (not nulltimeout) and (not err_handler) and aperiodic and enabled
             |)
          | (| return_code := (#INVALID_MODE when ((not enabled) or err_handler or (not aperiodic))) default (#
                              INVALID_PARAM when outofrange) default #NO_ERROR
             | return_code ^= timeout
             |)
          | (| PROCESS_SETSTATE{}(process_ID when OK_req,#WAITING)
             | on_counting := (when (not (timeout=INFINITE_TIME_VALUE))) when OK_req
             | START_COUNTER{}(process_ID when on_counting,timeout when on_counting)
             | diagnostic := PROCESS_SCHEDULINGREQUEST{}(when OK_req)
             |)
          |)/ aperiodic, err_handler, outofrange, enabled, process_ID, OK_req |)
      where 
      ProcessStatus_type status, status_IN, status_OUT;
      boolean nulltimeout, preemp_ON, outofrange, err_handler, aperiodic, OK_req, diagnostic, enabled;
      event on_counting;
      ProcessID_type process_ID;
      end
   ; 
   process SUSPEND =
        ( ? ProcessID_type process_ID;
          ! ReturnCode_type return_code;
        )
      (| (| (| (current_processID,status2) := PROCESS_GETACTIVE{}(^process_ID)
             | (present,status1) := PROCESS_CHECKID{}(process_ID when (^current_processID))
             | itself := (process_ID when (^current_processID))=current_processID
             | state := PROCESS_GETSTATE{}(process_ID when (^current_processID))
             | aperiodic := PROCESS_CHECKAPERIODIC{}(process_ID when (^current_processID))
             | err_handler := PROCESS_CHECKERRORHANDLER{}(process_ID when (^current_processID))
             | OK_req := (not itself) and (not (state=#DORMANT)) and aperiodic and present
             |)
          | (| return_code := (#INVALID_PARAM when ((not present) or itself)) default (#INVALID_MODE when ((state=#
                              DORMANT) or (not aperiodic))) default #NO_ERROR
             | return_code ^= process_ID
             |)
          | PROCESS_SETSTATE{}(process_ID when OK_req,#WAITING)
          |)/ aperiodic, state, itself, present, OK_req |)
      where 
      ProcessStatus_type status1, status2, status_OUT, status_IN;
      ProcessID_type current_processID;
      boolean err_handler, present, itself, aperiodic, OK_req;
      ProcessState_type state;
      end
   ; 
   process RESUME =
        ( ? ProcessID_type process_ID;
          ! ReturnCode_type return_code;
        )
      (| (| (| (current_processID,status2) := PROCESS_GETACTIVE{}(^process_ID)
             | (present,status1) := PROCESS_CHECKID{}(process_ID when (^current_processID))
             | itself := (process_ID when (^current_processID))=current_processID
             | state := PROCESS_GETSTATE{}(process_ID when (^current_processID))
             | OK_req := (not itself) and (not (state=#DORMANT)) and present
             |)
          | (| return_code := (#INVALID_PARAM when ((not present) or itself)) default (#INVALID_MODE when (state=#
                              DORMANT)) default #NO_ERROR
             | return_code ^= process_ID
             |)
          | (| STOP_COUNTER{}(process_ID when OK_req)
             | waitingonresource := PROCESS_CHECKWAITINDICATOR{}(process_ID when OK_req)
             | PROCESS_SETSTATE{}(process_ID when (not waitingonresource),#READY)
             | enabled := PROCESS_CHECKPREEMPTION{}(when (not waitingonresource))
             | diagnostic := PROCESS_SCHEDULINGREQUEST{}(when enabled)
             |)
          |)/ state, itself, present, OK_req |)
      where 
      ProcessStatus_type status1, status2, status_OUT, status_IN;
      ProcessID_type current_processID;
      boolean waitingonresource, present, itself, OK_req, diagnostic, enabled;
      ProcessState_type state;
      end
   ; 
   process STOP_SELF =
        ( )
      (| (process_ID,status) := PROCESS_GETACTIVE{}(true)
       | PROCESS_RELEASERESOURCES{}(process_ID)
       | PROCESS_SETSTATE{}(process_ID,#DORMANT)
       | err_handler := PROCESS_CHECKERRORHANDLER{}(process_ID)
       | enabled := PROCESS_CHECKPREEMPTION{}(^process_ID)
       | PROCESS_RESETLOCKLEVEL{}(^err_handler)
       | diagnostic := PROCESS_SCHEDULINGREQUEST{}(^err_handler)
       | PROCESS_LOCKLEVELINC{}((when (not enabled)) when err_handler)
       |)
      where 
      ProcessStatus_type status_OUT, status;
      ProcessID_type process_ID;
      boolean err_handler, preemp_ON, diagnostic, enabled;
      end
   ; 
   process STOP =
        ( ? ProcessID_type process_ID;
          ! ReturnCode_type return_code;
        )
      (| (| (| (current_processID,status2) := PROCESS_GETACTIVE{}(^process_ID)
             | (present,status1) := PROCESS_CHECKID{}(process_ID when (^current_processID))
             | itself := (process_ID when (^current_processID))=current_processID
             | state := PROCESS_GETSTATE{}(process_ID when (^current_processID))
             | OK_req := (not itself) and (not (state=#DORMANT)) and present
             |)
          | (| return_code := (#INVALID_PARAM when ((not present) or itself)) default (#NO_ACTION when (state=#DORMANT))
                              default #NO_ERROR
             | return_code ^= process_ID
             |)
          | (| PROCESS_RELEASERESOURCES{}(process_ID when OK_req)
             | PROCESS_SETSTATE{}(process_ID when OK_req,#DORMANT)
             | err_handler := PROCESS_CHECKERRORHANDLER{}(current_processID when OK_req)
             | preempted := PROCESS_PREVIOUSPREEMPTED{}(process_ID when err_handler)
             | preempted ^= when err_handler
             | PROCESS_RESETLOCKLEVEL{}(when preempted)
             | PROCESS_REMOVEWAITINGQUEUE{}(process_ID when OK_req)
             |)
          |)/ status, itself, present, current_processID, OK_req |)
      where 
      ProcessStatus_type status_OUT, status1, status2;
      ProcessState_type state, status_IN, status;
      boolean err_handler, present, itself, OK_req, preempted;
      ProcessID_type current_processID;
      end
   ; 
end %APEX_Process_Manager_lib%; 
