module APEX_QueuingPort_lib =

   process CREATE_QUEUING_PORT =
        ( ? Comm_ComponentName_type queuingPort_Name;
            QueuingPortSize_type queuingPort_Size;
            PortDirection_type port_direction;
            QueuingDiscipline_type queuing_discipline;
          ! Comm_ComponentID_type queuingPort_ID;
            ReturnCode_type return_code;
        )
      (| (| (| exceeded := QUEUINGPORT_CHECKCAPACITY{}()
             | exceeded ^= queuingPort_Name
             | (present,some_queuingPortID) := QUEUINGPORT_CHECKNAME{}(queuingPort_Name when (not exceeded))
             | created := QUEUINGPORT_CHECKCREATED{}(queuingPort_Name when present)
             | size_OK := (queuingPort_Size<=MAX_QUEUING_PORT_SIZE) when present
             | direction_OK := ((port_direction=#SOURCE) or (port_direction=#DESTINATION)) when present
             | queuing_OK := ((queuing_discipline=#FIFO) or (queuing_discipline=#PRIORITY)) when present
             | op_mode := PROCESS_CHECKOPERATINGMODE{}()
             | op_mode ^= when present
             | OK_req := size_OK and direction_OK and queuing_OK and (not created) and (not (op_mode=#NORMAL))
             |)
          | (| (queuingPort_ID,recorded) := QUEUINGPORT_RECORD{}(queuingPort_Name when OK_req,queuingPort_Size when 
                                             OK_req,port_direction when OK_req,MAX_MESSAGE_SIZE when OK_req,
                                             queuing_discipline when OK_req)
             | return_code := (#INVALID_CONFIG when (exceeded default (not present) default (not (size_OK and 
                              direction_OK and queuing_OK)))) default (#INVALID_MODE when (op_mode=#NORMAL)) default (#
                              NO_ACTION when created) default #NO_ERROR
             | return_code ^= queuingPort_Name ^= queuingPort_Size ^= port_direction ^= queuing_discipline
             |)
          |)/ OK_req, op_mode, queuing_OK, direction_OK, size_OK, created, present, exceeded |)
      where 
      Comm_ComponentID_type some_queuingPortID;
      boolean recorded, exceeded, present, created, size_OK, direction_OK, queuing_OK, OK_req;
      OperatingMode_type op_mode;
      end
   ; 
   process SEND_QUEUING_PORT =
        { ProcessID_type process_ID; }
        ( ? Comm_ComponentID_type queuingPort_ID;
            MessageArea_type message;
            MessageSize_type length;
            SystemTime_type timeout;
          ! ReturnCode_type return_code;
        )
      (| (| (| blocked := (queuingPort_ID ^- return_code) default (false when (^queuingPort_ID))
             | zblocked := blocked$1 init false
             | on_waiting := zblocked when (^queuingPort_ID)
             |)
          | return_code_1 := SEND{process_ID}(queuingPort_ID when (not on_waiting),message when (not on_waiting),length 
                                             when (not on_waiting),timeout when (not on_waiting))
          | (| queuingPort_ID ^= message ^= length ^= timeout
             | return_code := return_code_1 default (#NO_ERROR when on_waiting)
             |)
          |)/ return_code_1, on_waiting |)
      where 
      boolean blocked, zblocked, on_waiting;
      process SEND =
           { ProcessID_type process_ID; }
           ( ? Comm_ComponentID_type queuingPort_ID;
               MessageArea_type message;
               MessageSize_type length;
               SystemTime_type timeout;
             ! ReturnCode_type return_code;
           )
         (| (| (present,queuingPort_in) := QUEUINGPORT_CHECKID{}(queuingPort_ID)
             | (| outofrange := timeout>MAX_TIMEOUT_VALUE
                | available := timeout>0.0
                |)
             | (| (| (| length_OK := (length when present when (not outofrange))<=(queuingPort_in.Message_Size when (not
                                     outofrange))
                      | mess.Message_Area := message when length_OK
                      | mess.Message_Size := length when length_OK
                      | port_OK := (queuingPort_in.Port_Direction=#SOURCE) when length_OK
                      | sufficient_space := (queuingPort_in.Nb_Message<queuingPort_in.QueuingPort_Size) when length_OK
                      | nowaitingprocess := (queuingPort_in.WaitingOnReceive_Processes=0) when sufficient_space
                      | queuingPort_out := SET_MESSAGE{}(queuingPort_in when nowaitingprocess,mess when nowaitingprocess
                                           )
                      | err_handler := PROCESS_CHECKERRORHANDLER{}(process_ID when (not sufficient_space) when available
                                       )
                      | enabled := PROCESS_CHECKPREEMPTION{}(when (not err_handler))
                      |)
                   | port_OK_1 := port_OK
                   | err_handler_2 := err_handler
                   | sufficient_space_3 := sufficient_space
                   | enabled_4 := enabled
                   | length_OK_5 := length_OK
                   |) where boolean port_OK, err_handler, sufficient_space, enabled, length_OK; end
                | (| enabled := enabled_4
                   | (| PROCESS_SETSTATE{}(process_ID when enabled,#WAITING)
                      | INSERT_QUEUINGPORTSENDQUEUE{}(process_ID when enabled,mess when enabled,queuingPort_in when 
                                             enabled)
                      | START_COUNTER{}(process_ID when enabled when (not (timeout=INFINITE_TIME_VALUE)),timeout when 
                                        enabled when (not (timeout=INFINITE_TIME_VALUE)))
                      |)
                   |) where boolean enabled; end
                | (| some_processID := REMOVE_QUEUINGPORTRECEIVEQUEUE{}(queuingPort_in when (not nowaitingprocess))
                   | PROCESS_COPYMESSAGE(mess when (not nowaitingprocess),some_processID when (not nowaitingprocess))
                   | STOP_COUNTER{}(some_processID)
                   | PROCESS_SETSTATE{}(some_processID,#READY)
                   |)
                | (| enabled := enabled_4
                   | diagnostic := PROCESS_SCHEDULINGREQUEST{}(some_processID ^+ (when enabled))
                   |) where boolean enabled; end
                |)/ some_processID, nowaitingprocess, mess
             | (| length_OK := length_OK_5
                | enabled := enabled_4
                | sufficient_space := sufficient_space_3
                | err_handler := err_handler_2
                | port_OK := port_OK_1
                | (| return_code := (#INVALID_PARAM when ((not present) or outofrange)) default (#INVALID_CONFIG when (
                                    not length_OK)) default (#INVALID_MODE when (not port_OK)) default (#NOT_AVAILABLE 
                                    when (not sufficient_space) when (not available)) default (#INVALID_MODE when ((not 
                                    enabled) default err_handler)) default (#NO_ERROR when sufficient_space)
                   | queuingPort_ID ^= timeout ^= message ^= length
                   |)
                |) where boolean port_OK, err_handler, sufficient_space, enabled, length_OK; end
             |)/ length_OK_5, enabled_4, sufficient_space_3, err_handler_2, port_OK_1, available, outofrange, 
                 queuingPort_in, present |)
         where 
         APEX_QueuingPort_type queuingPort_out, queuingPort_in;
         boolean diagnostic, present, outofrange, available, port_OK_1, port_OK, err_handler_2, err_handler, 
                sufficient_space_3, sufficient_space, enabled_4, enabled, length_OK_5, length_OK, nowaitingprocess;
         Message_type mess;
         process SET_MESSAGE =
              ( ? APEX_QueuingPort_type queuingPort_IN;
                  Message_type message;
                ! APEX_QueuingPort_type queuingPort_OUT;
              )
           (| aux := ([{i to (MAX_QUEUING_PORT_SIZE-1)}:queuingPort_IN.Message_Range[i],
                            [ queuingPort_IN.Nb_Message]:message] when (queuingPort_IN.Nb_Message<queuingPort_IN.QueuingPort_Size)) 
% ( queuingPort_IN.Message_Range next [queuingPort_IN.Nb_Message]:message) when (queuingPort_IN.Nb_Message<queuingPort_IN.QueuingPort_Size)%
             | queuingPort_OUT.QueuingPort_Name := queuingPort_IN.QueuingPort_Name
             | queuingPort_OUT.QueuingPort_Size := queuingPort_IN.QueuingPort_Size
             | queuingPort_OUT.Port_Direction := queuingPort_IN.Port_Direction
             | queuingPort_OUT.Message_Size := queuingPort_IN.Message_Size
             | queuingPort_OUT.Nb_Message := ((queuingPort_IN.Nb_Message+1) when (queuingPort_IN.Nb_Message<
                                             queuingPort_IN.QueuingPort_Size)) default queuingPort_IN.Nb_Message
             | queuingPort_OUT.Message_Range := aux default queuingPort_IN.Message_Range
             | queuingPort_OUT.Queuing_Discipline := queuingPort_IN.Queuing_Discipline
             | queuingPort_OUT.WaitingOnSend_Processes := queuingPort_IN.WaitingOnSend_Processes
             | queuingPort_OUT.WaitingOnReceive_Processes := queuingPort_IN.WaitingOnReceive_Processes
             | queuingPort_OUT.WaitingOnSend_Queue := queuingPort_IN.WaitingOnSend_Queue
             | queuingPort_OUT.WaitingOnReceive_Queue := queuingPort_IN.WaitingOnReceive_Queue
             | QUEUINGPORT_UPDATE{}(queuingPort_OUT)
             |)
            where 
              [MAX_BUFFER_SIZE]Message_type aux;
            end;
         ProcessID_type some_processID;
         end
      ; 
      ReturnCode_type return_code_1;
      end
   ; 
   process RECEIVE_QUEUING_PORT =
        { ProcessID_type process_ID; }
        ( ? Comm_ComponentID_type queuingPort_ID;
            SystemTime_type timeout;
            MessageSize_type maximum_length;
          ! MessageArea_type message;
            MessageSize_type length;
            ReturnCode_type return_code;
        )
      (| (| (| blocked := (false when (^message_2)) default (queuingPort_ID ^- message_1) default (false when (^
                          queuingPort_ID))
             | zblocked := blocked$1 init false
             | retrieve := zblocked when (^queuingPort_ID)
             |)
          | (message_1,length_1,return_code_1) := RECEIVE{process_ID}(queuingPort_ID when (not retrieve),timeout when (
                                             not retrieve),maximum_length when (not retrieve))
          | message_2 := PROCESS_RETRIEVEMESSAGE{}(process_ID when retrieve)
          | (| message := message_2.Message_Area default message_1
             | length := message_2.Message_Size default length_1
             | queuingPort_ID ^= timeout ^= maximum_length
             | return_code := return_code_1 default (#NO_ERROR when (^message_2))
             |)
          |)/ message_2, return_code_1, length_1, message_1, retrieve |)
      where 
      boolean blocked, zblocked, retrieve;
      Message_type message_2;
      MessageArea_type message_1;
      MessageSize_type length_1;
      ReturnCode_type return_code_1;
      process RECEIVE =
           { ProcessID_type process_ID; }
           ( ? Comm_ComponentID_type queuingPort_ID;
               SystemTime_type timeout;
               MessageSize_type maximum_length;
             ! MessageArea_type message;
               integer length;
               ReturnCode_type return_code;
           )
         (| (| (present,queuingPort_in) := QUEUINGPORT_CHECKID{}(queuingPort_ID)
             | (| outofrange := timeout>MAX_TIMEOUT_VALUE
                | available := timeout>0.0
                |)
             | (| (| (| length_OK := (maximum_length when present when (not outofrange))<=(queuingPort_in.Message_Size 
                                     when (not outofrange))
                      | port_OK := (queuingPort_in.Port_Direction=#DESTINATION) when length_OK
                      | empty := (queuingPort_in.Nb_Message=0) when length_OK
                      | nowaitingprocess := (queuingPort_in.WaitingOnSend_Processes=0) when (not empty)
                      | (queuingPort_out,mess) := GET_MESSAGE{}(queuingPort_in when nowaitingprocess,var some_message)
                      | valid_size := mess.Message_Size<=(maximum_length when (^mess))
                      | length := mess.Message_Size when valid_size
                      | message := mess.Message_Area when valid_size
                      | err_handler := PROCESS_CHECKERRORHANDLER{}(process_ID when empty when available)
                      | enabled := PROCESS_CHECKPREEMPTION{}(when (not err_handler))
                      |)
                   | length_OK_1 := length_OK
                   | port_OK_2 := port_OK
                   | err_handler_3 := err_handler
                   | empty_4 := empty
                   | enabled_5 := enabled
                   | valid_size_6 := valid_size
                   |) where boolean length_OK, port_OK, err_handler, empty, enabled, valid_size; end
                | (| enabled := enabled_5
                   | (| PROCESS_SETSTATE{}(process_ID when enabled,#WAITING)
                      | INSERT_QUEUINGPORTRECEIVEQUEUE{}(process_ID when enabled,queuingPort_in when enabled)
                      | START_COUNTER{}(process_ID when enabled when (not (timeout=INFINITE_TIME_VALUE)),timeout when 
                                        enabled when (not (timeout=INFINITE_TIME_VALUE)))
                      |)
                   |) where boolean enabled; end
                | (| (some_processID,some_message) := REMOVE_QUEUINGPORTSENDQUEUE{}(queuingPort_in when (not 
                                             nowaitingprocess))
                   | STOP_COUNTER{}(some_processID)
                   | PROCESS_SETSTATE{}(some_processID,#READY)
                   |)
                | (| enabled := enabled_5
                   | diagnostic := PROCESS_SCHEDULINGREQUEST{}(some_processID ^+ (when enabled))
                   |) where boolean enabled; end
                |)/ some_message, some_processID, nowaitingprocess
             | (| valid_size := valid_size_6
                | enabled := enabled_5
                | empty := empty_4
                | err_handler := err_handler_3
                | port_OK := port_OK_2
                | length_OK := length_OK_1
                | (| return_code := (#INVALID_PARAM when ((not present) or outofrange)) default (#INVALID_CONFIG when ((
                                    not valid_size) default (not length_OK))) default (#INVALID_MODE when (not port_OK))
                                    default (#NOT_AVAILABLE when empty when (not available)) default (#INVALID_MODE when
                                    ((not enabled) default err_handler)) default (#NO_ERROR when (not empty))
                   | queuingPort_ID ^= timeout ^= maximum_length
                   |)
                |) where boolean length_OK, port_OK, err_handler, empty, enabled, valid_size; end
             |)/ valid_size_6, enabled_5, empty_4, err_handler_3, port_OK_2, length_OK_1, available, outofrange, 
                 queuingPort_in, present |)
         where 
         Message_type some_message, mess;
         APEX_QueuingPort_type queuingPort_out, queuingPort_in;
         boolean diagnostic, present, outofrange, available, length_OK_1, length_OK, port_OK_2, port_OK, err_handler_3, 
                err_handler, empty_4, empty, enabled_5, enabled, valid_size_6, valid_size, nowaitingprocess;
         process GET_MESSAGE =
              ( ? APEX_QueuingPort_type queuingPort_IN;
                  Message_type message_IN;
                ! APEX_QueuingPort_type queuingPort_OUT;
                  Message_type message_OUT;
              )
           (| (| aux :=  [{i to (MAX_QUEUING_PORT_SIZE-2)}:queuingPort_IN.Message_Range[i+1], 
                         [MAX_QUEUING_PORT_SIZE-1]:message_IN]  when (^message_IN) when ( queuingPort_IN.Nb_Message>0)
                 % array I to (MAX_QUEUING_PORT_SIZE-1) of
                 (| aux [I]:= ((queuingPort_IN.Message_Range[I+1])\\message_IN )  when ((^message_IN) when (queuingPort_IN.Nb_Message>0) )|)
               end %
                | queuingPort_OUT.QueuingPort_Name := queuingPort_IN.QueuingPort_Name
                | queuingPort_OUT.QueuingPort_Size := queuingPort_IN.QueuingPort_Size
                | queuingPort_OUT.Port_Direction := queuingPort_IN.Port_Direction
                | queuingPort_OUT.Message_Size := queuingPort_IN.Message_Size
                | queuingPort_OUT.Nb_Message := ((queuingPort_IN.Nb_Message-1) when (queuingPort_IN.Nb_Message>0)) 
                                             default queuingPort_IN.Nb_Message
                | queuingPort_OUT.Message_Range := aux default queuingPort_IN.Message_Range
                | queuingPort_OUT.Queuing_Discipline := queuingPort_IN.Queuing_Discipline
                | queuingPort_OUT.WaitingOnSend_Processes := queuingPort_IN.WaitingOnSend_Processes
                | queuingPort_OUT.WaitingOnReceive_Processes := queuingPort_IN.WaitingOnReceive_Processes
                | queuingPort_OUT.WaitingOnSend_Queue := queuingPort_IN.WaitingOnSend_Queue
                | queuingPort_OUT.WaitingOnReceive_Queue := queuingPort_IN.WaitingOnReceive_Queue
                | QUEUINGPORT_UPDATE{}(queuingPort_OUT)
                |)
             | (| mess := queuingPort_OUT.Message_Range[(MAX_QUEUING_PORT_SIZE-queuingPort_OUT.Nb_Message)-1]
                | message_OUT := mess when (queuingPort_IN.Nb_Message>0)
                |)
             |)
            where 
              Message_type mess;
              [MAX_BUFFER_SIZE]Message_type aux;
            end
         ; 
         ProcessID_type some_processID;
         end
      ; 
      end
   ; 
   process GET_QUEUING_PORT_ID =
        ( ? Comm_ComponentName_type queuingPort_Name;
          ! Comm_ComponentID_type queuingPort_ID;
            ReturnCode_type return_code;
        )
      (| (| (present,queuingPort_ID) := QUEUINGPORT_CHECKNAME{}(queuingPort_Name)
          | (| return_code := (#INVALID_CONFIG when (not present)) default #NO_ERROR
             | queuingPort_Name ^= return_code
             |)
          |)/ present |)
      where 
      boolean present;
      end
   ; 
   process GET_QUEUING_PORT_STATUS =
        ( ? Comm_ComponentID_type queuingPort_ID;
          ! QueuingPortStatus_type status;
            ReturnCode_type return_code;
        )
      (| (| (present,queuingPort_in) := QUEUINGPORT_CHECKID{}(queuingPort_ID)
          | (| return_code := (#INVALID_PARAM when (not present)) default #NO_ERROR
             | return_code ^= queuingPort_ID
             |)
          | (| status.QueuingPort_Size := queuingPort_in.QueuingPort_Size
             | status.Port_Direction := queuingPort_in.Port_Direction
             | status.Nb_Message := queuingPort_in.Nb_Message
             | status.Waiting_Processes := queuingPort_in.WaitingOnSend_Processes+queuingPort_in.
                                           WaitingOnReceive_Processes
             |)
          |)/ present, queuingPort_in |)
      where 
      APEX_QueuingPort_type queuingPort_in;
      boolean present;
      end
   ; 
end %APEX_QueuingPort_lib%; 
