module APEX_SamplingPort_lib =

   process CREATE_SAMPLING_PORT =
        ( ? Comm_ComponentName_type samplingPort_Name;
            SamplingPortSize_type samplingPort_Size;
            PortDirection_type port_direction;
            SystemTime_type refresh_period;
          ! Comm_ComponentID_type samplingPort_ID;
            ReturnCode_type return_code;
        )
     (| (| exceeded := SAMPLINGPORT_CHECKCAPACITY{}()
         | exceeded ^= samplingPort_Name
         | (present,some_samplingPort) := SAMPLINGPORT_CHECKNAME{}(samplingPort_Name when (not exceeded))
         | created := SAMPLINGPORT_CHECKCREATED{}(samplingPort_Name when present)
         | size_OK := (samplingPort_Size<=MAX_SAMPLING_PORT_SIZE) when present
         | direction_OK := ((port_direction=#SOURCE) or (port_direction=#DESTINATION)) when present
         | period_OK := (refresh_period<=MAX_PERIOD_VALUE) when present
         | op_mode := PROCESS_CHECKOPERATINGMODE{}()
         | op_mode ^= when present
         | b := size_OK and direction_OK and period_OK
         | OK_req := b and (not created) and (not (op_mode=#NORMAL))
         |)
      | (| (samplingPort_ID,recorded) := SAMPLINGPORT_RECORD{}(samplingPort_Name when OK_req,samplingPort_Size when OK_req,
                                                               port_direction when OK_req,refresh_period when OK_req)
         | return_code := (#INVALID_CONFIG when (exceeded default (not present) default not b))
                          default (#INVALID_MODE when (op_mode=#NORMAL))
                          default (#NO_ACTION when created) 
                          default #NO_ERROR
         | return_code ^= samplingPort_Name ^= samplingPort_Size ^= port_direction ^= refresh_period
         |)
      |)
     where 
       boolean b;
       Comm_ComponentID_type some_samplingPort;
       boolean recorded, exceeded, present, created, size_OK, direction_OK, period_OK, OK_req;
       OperatingMode_type op_mode;
     end
 ; 
   process WRITE_SAMPLING_PORT =
        ( ? Comm_ComponentID_type samplingPort_ID;
            MessageArea_type message;
            MessageSize_type length;
          ! ReturnCode_type return_code;
        )
     (| (| (present,samplingPort_in) := SAMPLINGPORT_CHECKID{}(samplingPort_ID)
         | length_OK := (length when present)<=samplingPort_in.SamplingPort_Size
         | mess.Message_Area := message when length_OK
         | mess.Message_Size := length when length_OK
         | port_OK := samplingPort_in.Port_Direction=#SOURCE
         | OK_req := (when length_OK) when port_OK
         |)
      | (| samplingPort_out := WRITESAMPLING{}(samplingPort_in when OK_req,mess when OK_req)
         | return_code := (#INVALID_PARAM when (not present)) 
                          default (#INVALID_CONFIG when (not length_OK))
                          default (#INVALID_MODE when (not port_OK))
                          default #NO_ERROR
         | return_code ^= samplingPort_ID ^= length ^= message
         |)
      |)
      where 
        APEX_SamplingPort_type samplingPort_out, samplingPort_in;
        process WRITESAMPLING =
          ( ? APEX_SamplingPort_type samplingPort_IN;
              Message_type message;
            ! APEX_SamplingPort_type samplingPort_OUT;
              )
          (| samplingPort_OUT.SamplingPort_Name := samplingPort_IN.SamplingPort_Name
           | samplingPort_OUT.SamplingPort_Size := samplingPort_IN.SamplingPort_Size
           | samplingPort_OUT.Port_Direction := samplingPort_IN.Port_Direction
           | samplingPort_OUT.Refresh_Period := samplingPort_IN.Refresh_Period
           | samplingPort_OUT.Validity := samplingPort_IN.Validity
           | samplingPort_OUT.Message := message
           | samplingPort_OUT.Empty := (samplingPort_IN.Empty when (not samplingPort_IN.Empty))
                                       default (not samplingPort_IN.Empty)
           | samplingPort_OUT.Message_Age := 0.0
           | SAMPLINGPORT_UPDATE{}(samplingPort_OUT)
           |)
        ; 
        Message_type mess;
        event OK_req;
        boolean present, length_OK, port_OK;
      end
   ; 
   process READ_SAMPLING_PORT =
        ( ? Comm_ComponentID_type samplingPort_ID;
          ! MessageArea_type message;
            MessageSize_type length;
            boolean validity;
            ReturnCode_type return_code;
        )
     (| (| (present,samplingPort_in) := SAMPLINGPORT_CHECKID{}(samplingPort_ID)
         | port_OK := samplingPort_in.Port_Direction=#DESTINATION
         |)
      | (| validity := ((false when (samplingPort_in.Empty or (samplingPort_in.Message_Age>samplingPort_in.Refresh_Period)))
                       default (true when (samplingPort_in.Message_Age<=samplingPort_in.Refresh_Period))) when port_OK
         | message := samplingPort_in.Message.Message_Area when (not samplingPort_in.Empty) when port_OK
         | length := samplingPort_in.Message.Message_Size when (not samplingPort_in.Empty) when port_OK
         | return_code := (#INVALID_PARAM when (not present)) default (#INVALID_MODE when (not port_OK)) default #NO_ERROR
         | return_code ^= samplingPort_ID
         |)
      |)
     where 
       APEX_SamplingPort_type samplingPort_in;
       boolean present, port_OK;
     end
       ; 
   process GET_SAMPLING_PORT_ID =
        ( ? Comm_ComponentName_type samplingPort_Name;
          ! Comm_ComponentID_type samplingPort_ID;
            ReturnCode_type return_code;
        )
      (| (present,samplingPort_ID) := SAMPLINGPORT_CHECKNAME{}(samplingPort_Name)
       | (| return_code := (#INVALID_CONFIG when (not present)) default #NO_ERROR
          | samplingPort_Name ^= return_code
          |)
       |)
      where 
        boolean present;
      end
   ; 
   process GET_SAMPLING_PORT_STATUS =
        ( ? Comm_ComponentID_type samplingPort_ID;
          ! SamplingPortStatus_type status;
            ReturnCode_type return_code;
        )
     (| (present,samplingPort_in) := SAMPLINGPORT_CHECKID{}(samplingPort_ID)
      | (| status.SamplingPort_Size := samplingPort_in.SamplingPort_Size
         | status.Port_Direction := samplingPort_in.Port_Direction
         | status.Refresh_Period := samplingPort_in.Refresh_Period
         | status.Validity := samplingPort_in.Validity
         |)
      | (| return_code := (#INVALID_PARAM when (not present)) default #NO_ERROR
         | return_code ^= samplingPort_ID
         |)
      |)
     where 
       APEX_SamplingPort_type samplingPort_in;
       boolean present;
     end
       ; 
     end %APEX_SamplingPort_lib%; 
