module APEX_Semaphore_lib =

   process CREATE_SEMAPHORE =
        ( ? Comm_ComponentName_type semaphore_name;
            SemaphoreValue_type current_value;
            SemaphoreValue_type maximum_value;
            QueuingDiscipline_type queuing_discipline;
          ! Comm_ComponentID_type semaphore_ID;
            ReturnCode_type return_code;
        )
      (| (| (| full := SEMAPHORE_CHECKCAPACITY{}()
             | (present,some_semaphoreID) := SEMAPHORE_CHECKNAME{}(semaphore_name)
             | op_mode := PROCESS_CHECKOPERATINGMODE{}()
             | valid_param := (current_value<MAX_SEMAPHORE_VALUE) and (maximum_value<MAX_SEMAPHORE_VALUE) and (
                              current_value<=maximum_value) and ((queuing_discipline=#FIFO) or (queuing_discipline=#
                              PRIORITY))
             | OK_perform := (when (not full)) when (not present) when (not (op_mode=#NORMAL)) when valid_param
             |)
          | (| (semaphore_ID,recorded) := SEMAPHORE_RECORD{}(semaphore_name when OK_perform,current_value when 
                                             OK_perform,maximum_value when OK_perform,queuing_discipline when OK_perform
                                          )
             | return_code := (#INVALID_CONFIG when full) default (#NO_ACTION when present) default (#INVALID_PARAM when
                              (not valid_param)) default (#INVALID_MODE when (op_mode=#NORMAL)) default #NO_ERROR
             | return_code ^= semaphore_name ^= current_value ^= maximum_value ^= queuing_discipline ^= full ^= op_mode
             |)
          |)/ valid_param, op_mode, present, full, OK_perform |)
      where 
      Comm_ComponentID_type some_semaphoreID;
      boolean recorded, full, present, valid_param;
      event OK_perform;
      OperatingMode_type op_mode;
      end
   ; 
   process WAIT_SEMAPHORE =
        { ProcessID_type process_ID; }
        ( ? Comm_ComponentID_type semaphore_ID;
            SystemTime_type timeout;
          ! ReturnCode_type return_code;
        )
      (| (| (| blocked := (semaphore_ID ^- return_code) default (false when (^semaphore_ID))
             | zblocked := blocked$1 init false
             | on_waiting := zblocked when (^semaphore_ID)
             |)
          | return_code_1 := WAIT{process_ID}(semaphore_ID when (not on_waiting),timeout when (not on_waiting))
          | (| semaphore_ID ^= timeout
             | return_code := return_code_1 default (#NO_ERROR when on_waiting)
             |)
          |)/ return_code_1, on_waiting |)
      where 
      boolean blocked, zblocked, on_waiting;
      process WAIT =
           { ProcessID_type process_ID; }
           ( ? Comm_ComponentID_type semaphore_ID;
               SystemTime_type timeout;
             ! ReturnCode_type return_code;
           )
         (| (| (present,sema_in) := SEMAPHORE_CHECKID{}(semaphore_ID)
             | (| outofrange := timeout>MAX_TIMEOUT_VALUE
                | available := timeout>0.0
                |)
             | (| (| (| positive := (sema_in.Current_Value>0) when (not outofrange)
                      | sema_out := WAITSEM{}(sema_in when positive)
                      | err_handler := PROCESS_CHECKERRORHANDLER{}(process_ID when (not positive) when available)
                      | enabled := PROCESS_CHECKPREEMPTION{}(when (not err_handler))
                      | PROCESS_SETSTATE{}(process_ID when enabled,#WAITING)
                      | INSERT_SEMAPHOREQUEUE{}(process_ID when enabled,sema_in when enabled)
                      | START_COUNTER{}(process_ID when enabled when (not (timeout=INFINITE_TIME_VALUE)),timeout when 
                                        enabled when (not (timeout=INFINITE_TIME_VALUE)))
                      |)
                   | err_handler_1 := err_handler
                   | positive_2 := positive
                   | enabled_3 := enabled
                   |) where boolean err_handler, positive, enabled; end
                | (| enabled := enabled_3
                   | diagnostic := PROCESS_SCHEDULINGREQUEST{}(when enabled)
                   |) where boolean enabled; end
                |)
             | (| enabled := enabled_3
                | positive := positive_2
                | err_handler := err_handler_1
                | (| return_code := (#INVALID_PARAM when ((not present) or outofrange)) default (#NOT_AVAILABLE when (
                                    not positive) when (not available)) default (#INVALID_MODE when ((not enabled) 
                                    default err_handler)) default (#NO_ERROR when positive)
                   | semaphore_ID ^= timeout
                   |)
                |) where boolean err_handler, positive, enabled; end
             |)/ enabled_3, positive_2, err_handler_1, outofrange, available, sema_in, present |)
         where 
         boolean present, outofrange, positive, preemp_OK, available, err_handler_1, err_handler, positive_3, 
                preemp_ON_4, preemp_ON, diagnostic, enabled, diagnostic_4, enabled_4, positive_2, enabled_3;
         APEX_Semaphore_type sema_out, sema_in;
         process WAITSEM =
              ( ? APEX_Semaphore_type semaphore_IN;
                ! APEX_Semaphore_type semaphore_OUT;
              )
            (| semaphore_OUT.Semaphore_Name := semaphore_IN.Semaphore_Name
             | semaphore_OUT.Current_Value := ((semaphore_IN.Current_Value-1) when (semaphore_IN.Current_Value>0)) 
                                             default semaphore_IN.Current_Value
             | semaphore_OUT.Maximum_Value := semaphore_IN.Maximum_Value
             | semaphore_OUT.Waiting_Processes := semaphore_IN.Waiting_Processes
             | semaphore_OUT.Queuing_Discipline := semaphore_IN.Queuing_Discipline
             | semaphore_OUT.Waiting_Queue := semaphore_IN.Waiting_Queue
             | SEMAPHORE_UPDATE{}(semaphore_OUT)
             |)
         ; 
         end
      ; 
      ReturnCode_type return_code_1;
      end
   ; 
   process SIGNAL_SEMAPHORE =
        ( ? Comm_ComponentID_type semaphore_ID;
          ! ReturnCode_type return_code;
        )
      (| (| (present,sema_in) := SEMAPHORE_CHECKID{}(semaphore_ID)
          | (| (| (| noaction := sema_in.Current_Value=sema_in.Maximum_Value
                   | release := sema_in.Waiting_Processes>0
                   | sema_out := SIGNALSEM{}(sema_in when (not release) when (not noaction))
                   | some_processID := REMOVE_SEMAPHOREQUEUE{}(sema_in when release when (not noaction))
                   | STOP_COUNTER{}(some_processID)
                   | PROCESS_SETSTATE{}(some_processID,#READY)
                   | enabled := PROCESS_CHECKPREEMPTION{}(when (^some_processID))
                   |)
                | noaction_1 := noaction
                |) where boolean noaction; end
             | diagnostic := PROCESS_SCHEDULINGREQUEST{}(when enabled)
             |)/ enabled
          | (| noaction := noaction_1
             | (| return_code := (#INVALID_PARAM when (not present)) default (#NO_ACTION when noaction) default #
                                 NO_ERROR
                | semaphore_ID ^= return_code
                |)
             |) where boolean noaction; end
          |)/ noaction_1, sema_in, present |)
      where 
      ProcessID_type some_processID;
      boolean preemp_OK, release, present, noaction, noaction_1, diagnostic, enabled;
      APEX_Semaphore_type sema_out, sema_in;
      process SIGNALSEM =
           ( ? APEX_Semaphore_type semaphore_IN;
             ! APEX_Semaphore_type semaphore_OUT;
           )
         (| semaphore_OUT.Semaphore_Name := semaphore_IN.Semaphore_Name
          | semaphore_OUT.Current_Value := ((semaphore_IN.Current_Value+1) when (semaphore_IN.Current_Value<semaphore_IN
                                           .Maximum_Value)) default semaphore_IN.Current_Value
          | semaphore_OUT.Maximum_Value := semaphore_IN.Maximum_Value
          | semaphore_OUT.Waiting_Processes := semaphore_IN.Waiting_Processes
          | semaphore_OUT.Queuing_Discipline := semaphore_IN.Queuing_Discipline
          | semaphore_OUT.Waiting_Queue := semaphore_IN.Waiting_Queue
          | SEMAPHORE_UPDATE{}(semaphore_OUT)
          |)
      ; 
      end
   ; 
   process GET_SEMAPHORE_ID =
        ( ? Comm_ComponentName_type semaphore_name;
          ! Comm_ComponentID_type semaphore_ID;
            ReturnCode_type return_code;
        )
      (| (| (| return_code := (#INVALID_CONFIG when (not present)) default #NO_ERROR
             | semaphore_name ^= return_code
             |)
          | (present,semaphore_ID) := SEMAPHORE_CHECKNAME{}(semaphore_name)
          |)/ present |)
      where 
      boolean present;
      end
   ; 
   process GET_SEMAPHORE_STATUS =
        ( ? Comm_ComponentID_type semaphore_ID;
          ! SemaphoreStatus_type semaphore_status;
            ReturnCode_type return_code;
        )
      (| (| (present,sema_in) := SEMAPHORE_CHECKID{}(semaphore_ID)
          | (| return_code := (#INVALID_PARAM when (not present)) default #NO_ERROR
             | return_code ^= semaphore_ID
             |)
          | (| semaphore_status.Current_Value := sema_in.Current_Value
             | semaphore_status.Maximum_Value := sema_in.Maximum_Value
             | semaphore_status.Waiting_Processes := sema_in.Waiting_Processes
             |)
          |)/ present, sema_in |)
      where 
      APEX_Semaphore_type sema_in;
      boolean present;
      end
   ; 
end %APEX_Semaphore_lib%; 
