module APEX_Time_Manager_lib =
   process GET_TIME =
        ( ! SystemTime_type system_time;
            ReturnCode_type return_code;
        )
      (| system_time := prag_GETTIME{}()
       | return_code := #NO_ERROR
       | return_code ^= system_time
       |)
      where 
      process prag_GETTIME =
           ( ! SystemTime_type system_time;
           )
         pragmas 
            C_Code "&o1 = GLOBAL_PROCESS_MANAGER->ProcessGetTime()"
         end pragmas
      
      ; 
      end
   ; 
   process TIMED_WAIT =
        ( ? SystemTime_type delay_time;
          ! ReturnCode_type return_code;
        )
      (| (| (| (process_ID,status) := PROCESS_GETACTIVE{}(^delay_time)
             | err_handler := PROCESS_CHECKERRORHANDLER{}(process_ID)
             | enabled := PROCESS_CHECKPREEMPTION{}(^process_ID)
             | outofrange := (delay_time>MAX_TIMEOUT_VALUE) when (^process_ID)
             | infinite := (delay_time=INFINITE_TIME_VALUE) when (^process_ID)
             | OK_req := (not outofrange) and (not infinite) and (not err_handler) and enabled
             |)
          | (| nulldelay := (delay_time=0.0) when OK_req
             | e1 :: PROCESS_SETSTATE{}(process_ID when nulldelay,#READY)
             | e2 :: PROCESS_SETSTATE{}(process_ID when (not nulldelay),#WAITING)
             | START_COUNTER{}(process_ID when (not nulldelay),delay_time when (not nulldelay))
             | e3 :: diagnostic := PROCESS_SCHEDULINGREQUEST{}(when OK_req)
             | e2 --> e3
             | e1 --> e3
             |)
          | (| return_code := (#INVALID_MODE when ((not enabled) or err_handler)) default (#INVALID_PARAM when (outofrange or infinite)) default #NO_ERROR
             | return_code ^= delay_time
             |)
          |)/ outofrange, err_handler, infinite, enabled, process_ID, OK_req |)
      where 
      boolean nulldelay, OK_req, enabled_1, enabled, infinite, err_handler, outofrange, diagnostic;
      ProcessStatus_type status;
      label e1, e2, e3; 
      ProcessID_type process_ID;
      end
   ; 
   process PERIODIC_WAIT =
        ( ! ReturnCode_type return_code;
        )
      (| (| (| (process_ID,status) := PROCESS_GETACTIVE{}(true)
             | aperiodic := PROCESS_CHECKAPERIODIC{}(process_ID)
             | enabled := PROCESS_CHECKPREEMPTION{}(^process_ID)
             | err_handler := PROCESS_CHECKERRORHANDLER{}(process_ID)
             | OK_req := (not err_handler) and enabled and (not aperiodic)
             | process_ID ^= OK_req
             |)
          | (| PROCESS_SETSTATE{}(process_ID when OK_req,#WAITING)
             | prag_PERIODICWAIT{}(process_ID when OK_req)
             | diagnostic := PROCESS_SCHEDULINGREQUEST{}(when OK_req)
             |)
          | (| return_code := (#INVALID_MODE when ((not enabled) or err_handler or aperiodic)) default #NO_ERROR
             | return_code ^= enabled ^= err_handler ^= aperiodic
             |)
          |)/ aperiodic, err_handler, enabled, process_ID, OK_req |)
      where 
      ProcessStatus_type status;
      boolean diagnostic, OK_req, enabled, err_handler, aperiodic;
      process prag_PERIODICWAIT =
           ( ? ProcessID_type process_ID;
           )
         pragmas 
            C_Code "GLOBAL_PROCESS_MANAGER->updateTimePeriodicWait(&i1) "
         end pragmas
      
      ; 
      ProcessID_type process_ID;
      end
   ; 
   process REPLENISH =
        ( ! ReturnCode_type return_code;
        )
      (| (process_ID,status) := PROCESS_GETACTIVE{}(true)
       | prag_REPLENISH{}(process_ID)
       | return_code := #NO_ERROR when (^process_ID)
       |)
      where 
      ProcessStatus_type status;
      ProcessID_type process_ID;
      process prag_REPLENISH =
           ( ? ProcessID_type process_ID;
           )
         pragmas 
            C_Code "GLOBAL_PROCESS_MANAGER->updateTimeReplenish(&i1)"
         end pragmas
      
      ; 
      end
   ; 
end %APEX_Time_Manager_lib%; 
