module % APEX_%InterCommunications_Manager_lib =

   process QUEUINGPORT_CHECKID =
        ( ? Comm_ComponentID_type queuingPort_ID;
          ! boolean present;
            APEX_QueuingPort_type queuingPort_OUT;
        )
        spec (| (| queuingPort_ID --> present
                 | {queuingPort_ID --> queuingPort_OUT} when present
                 |)
              | (| queuingPort_ID ^= present
                 | queuingPort_OUT ^= when present
                 |)
              |)
      pragmas 
         C_Code "&o1 = GLOBAL_QUEUINGPORT_MANAGER->QueuingPortCheckID(&i1, &o2)"
      end pragmas
   
   ; 
   process QUEUINGPORT_CHECKNAME =
        ( ? Comm_ComponentName_type queuingPort_name;
          ! boolean present;
            Comm_ComponentID_type queuingPort_ID;
        )
        spec (| (| queuingPort_name --> present
                 | {queuingPort_name --> queuingPort_ID} when present
                 |)
              | (| queuingPort_name ^= present
                 | queuingPort_ID ^= when present
                 |)
              |)
      pragmas 
         C_Code "&o1 = GLOBAL_QUEUINGPORT_MANAGER->QueuingPortCheckName(&i1, &o2)"
      end pragmas
   
   ; 
   process QUEUINGPORT_RECORD =
        ( ? Comm_ComponentName_type queuingPort_name;
            QueuingPortSize_type queuingPort_size;
            PortDirection_type port_direction;
            MessageSize_type message_size;
            QueuingDiscipline_type queuing_discipline;
          ! Comm_ComponentID_type queuingPort_ID;
            boolean recorded;
        )
        spec (| (| {queuingPort_name,queuingPort_size,port_direction,message_size,queuing_discipline} --> recorded
                 | {{queuingPort_name,queuingPort_size,port_direction,message_size,queuing_discipline} --> 
                    queuingPort_ID} when recorded
                 |)
              | (| queuingPort_name ^= queuingPort_size ^= port_direction ^= message_size ^= queuing_discipline ^= 
                   recorded
                 | queuingPort_ID ^= when recorded
                 |)
              |)
      pragmas 
         C_Code "&o2 = GLOBAL_QUEUINGPORT_MANAGER->QueuingPortRecord(&i1, &i2, &i3, &i4, &i5, &o1);"
      end pragmas
   
   ; 
   process QUEUINGPORT_UPDATE =
        ( ? APEX_QueuingPort_type queuingPort_IN;
        )
      pragmas 
         C_Code "GLOBAL_QUEUINGPORT_MANAGER->QueuingPortUpdate(&i1)"
      end pragmas
   
   ; 
   process SAMPLINGPORT_CHECKID =
        ( ? Comm_ComponentID_type samplingPort_ID;
          ! boolean present;
            APEX_SamplingPort_type samplingPort_OUT;
        )
        spec (| (| samplingPort_ID --> present
                 | {samplingPort_ID --> samplingPort_OUT} when present
                 |)
              | (| samplingPort_ID ^= present
                 | samplingPort_OUT ^= when present
                 |)
              |)
      pragmas 
         C_Code "&o1 = GLOBAL_SAMPLINGPORT_MANAGER->SamplingPortCheckID(&i1, &o2)"
      end pragmas
   
   ; 
   process SAMPLINGPORT_CHECKNAME =
        ( ? Comm_ComponentName_type samplingPort_name;
          ! boolean present;
            Comm_ComponentID_type samplingPort_ID;
        )
        spec (| (| samplingPort_name --> present
                 | {samplingPort_name --> samplingPort_ID} when present
                 |)
              | (| samplingPort_name ^= present
                 | samplingPort_ID ^= when present
                 |)
              |)
      pragmas 
         C_Code "&o1 = GLOBAL_SAMPLINGPORT_MANAGER->SamplingPortCheckName(&i1, &o2)"
      end pragmas
   
   ; 
   process SAMPLINGPORT_RECORD =
        ( ? Comm_ComponentName_type samplingPort_name;
            SamplingPortSize_type samplingPort_size;
            PortDirection_type port_direction;
            SystemTime_type refresh_period;
          ! Comm_ComponentID_type samplingPort_ID;
            boolean recorded;
        )
        spec (| (| {samplingPort_name,samplingPort_size,port_direction,refresh_period} --> recorded
                 | {{samplingPort_name,samplingPort_size,port_direction,refresh_period} --> samplingPort_ID} when 
                                             recorded
                 |)
              | (| samplingPort_name ^= samplingPort_size ^= port_direction ^= refresh_period ^= recorded
                 | samplingPort_ID ^= when recorded
                 |)
              |)
      pragmas 
         C_Code "&o2 = GLOBAL_SAMPLINGPORT_MANAGER->SamplingPortRecord(&i1,&i2, &i3, &i4,&o1)"
      end pragmas
   
   ; 
   process SAMPLINGPORT_UPDATE =
        ( ? APEX_SamplingPort_type samplingPort_IN;
        )
      pragmas 
         C_Code "GLOBAL_SAMPLINGPORT_MANAGER->SamplingPortUpdate(&i1)"
      end pragmas
   
   ; 
   process QUEUINGPORT_CHECKCAPACITY =
        ( ! boolean full;
        )
      pragmas 
         C_Code "&o1 = GLOBAL_QUEUINGPORT_MANAGER->Full()"
      end pragmas
   
   ; 
   process SAMPLINGPORT_CHECKCAPACITY =
        ( ! boolean full;
        )
      pragmas 
         C_Code "&o1 = GLOBAL_SAMPLINGPORT_MANAGER->Full()"
      end pragmas
   
   ; 
   process QUEUINGPORT_CHECKCREATED =
        ( ? Comm_ComponentName_type queuingPort_name;
          ! boolean created;
        )
        spec (| (| queuingPort_name --> created |)
              | (| queuingPort_name ^= created |)
              |)
      pragmas 
         C_Code "&o1 = GLOBAL_QUEUINGPORT_MANAGER->QueuingPortCheckCreated(&i1)"
      end pragmas
   
   ; 
   process SAMPLINGPORT_CHECKCREATED =
        ( ? Comm_ComponentName_type samplingPort_name;
          ! boolean created;
        )
        spec (| (| samplingPort_name --> created |)
              | (| samplingPort_name ^= created |)
              |)
      pragmas 
         C_Code "&o1 = GLOBAL_SAMPLINGPORT_MANAGER->SamplingPortCheckCreated(&i1)"
      end pragmas
   
   ; 
end %APEX_InterCommunications_Manager_lib%; 
