module PRIORITY_WaitingQueue_lib =
   process INSERT_BLACKBOARDQUEUE =
        ( ? ProcessID_type process_ID;
            APEX_Blackboard_type blackboard_IN;
        )
        spec (| process_ID ^= blackboard_IN |)
      pragmas 
         C_Code "&i2.Waiting_Queue[&i1] = TRUE;"
                "&i2.Waiting_Processes = (&i2.Waiting_Processes)+1; "
                "GLOBAL_BLACKBOARD_MANAGER->BlackboardUpdate(&i2)"
      end pragmas
   
   ; 
   process INSERT_BUFFERRECEIVEQUEUE =
        ( ? ProcessID_type process_ID;
            APEX_Buffer_type buffer_IN;
        )
        spec (| process_ID ^= buffer_IN |)
      pragmas 
         C_Code "if (&i2.Queuing_Discipline == FIFO) {((ptFIFO_ReceiveQueue)"
                "(&i2.WaitingOnReceive_Queue))->insert(&i1);} else {((ptPRIORITY_ReceiveQueue)"
                "(&i2.WaitingOnReceive_Queue))->insert(&i1);};"
                "&i2.WaitingOnReceive_Processes = "
                "(&i2.WaitingOnReceive_Processes) +1; "
                "GLOBAL_BUFFER_MANAGER->BufferUpdate(&i2)"
      end pragmas
   
   ; 
   process INSERT_EVENTQUEUE =
        ( ? ProcessID_type process_ID;
            APEX_Event_type event_IN;
        )
        spec (| process_ID ^= event_IN |)
      pragmas 
         C_Code "&i2.Waiting_Queue[&i1] = TRUE;"
                "&i2.Waiting_Processes = (&i2.Waiting_Processes) +1;"
                "GLOBAL_EVENT_MANAGER->EventUpdate(&i2)"
      end pragmas
   
   ; 
   process INSERT_SEMAPHOREQUEUE =
        ( ? ProcessID_type process_ID;
            APEX_Semaphore_type semaphore_IN;
        )
        spec (| process_ID ^= semaphore_IN |)
      pragmas 
         C_Code "if (&i2.Queuing_Discipline == FIFO) {((ptFIFO_ReceiveQueue)"
                "(&i2.Waiting_Queue))->insert(&i1);} else {((ptPRIORITY_ReceiveQueue)"
                "(&i2.Waiting_Queue))->insert(&i1);};"
                "&i2.Waiting_Processes = "
                "(&i2.Waiting_Processes) +1; "
                "GLOBAL_SEMAPHORE_MANAGER->SemaphoreUpdate(&i2)"
      end pragmas
   
   ; 
   process INSERT_BUFFERSENDQUEUE =
        ( ? ProcessID_type process_ID;
            Message_type message;
            APEX_Buffer_type buffer_IN;
        )
        spec (| process_ID ^= message ^= buffer_IN |)
      pragmas 
         C_Code "if (&i3.Queuing_Discipline == FIFO) {((ptFIFO_SendQueue)"
                "(&i3.WaitingOnSend_Queue))->insert(&i1, &i2);} else {((ptPRIORITY_SendQueue)"
                "(&i3.WaitingOnSend_Queue))->insert(&i1, &i2);};"
                "&i3.WaitingOnSend_Processes = "
                "(&i3.WaitingOnSend_Processes) +1; "
                "GLOBAL_BUFFER_MANAGER->BufferUpdate(&i3)"
      end pragmas
   
   ; 
   process REMOVE_BUFFERRECEIVEQUEUE =
        ( ? APEX_Buffer_type buffer_IN;
          ! ProcessID_type process_ID;
        )
        spec (| (| buffer_IN --> process_ID |)
              | (| process_ID ^= buffer_IN |)
              |)
      pragmas 
         C_Code "if (&i1.Queuing_Discipline == FIFO) {((ptFIFO_ReceiveQueue)"
                "(&i1.WaitingOnReceive_Queue))->remove(&o1);} else {((ptPRIORITY_ReceiveQueue)"
                "(&i1.WaitingOnReceive_Queue))->remove(&o1);};"
                "&i1.WaitingOnReceive_Processes = "
                "(&i1.WaitingOnReceive_Processes) -1; "
                "GLOBAL_BUFFER_MANAGER->BufferUpdate(&i1)"
      end pragmas
   
   ; 
   process REMOVE_SEMAPHOREQUEUE =
        ( ? APEX_Semaphore_type semaphore_IN;
          ! ProcessID_type process_ID;
        )
        spec (| (| semaphore_IN --> process_ID |)
              | (| process_ID ^= semaphore_IN |)
              |)
      pragmas 
         C_Code "if (&i1.Queuing_Discipline == FIFO) {((ptFIFO_ReceiveQueue)"
                "(&i1.Waiting_Queue))->remove(&o1);} else {((ptPRIORITY_ReceiveQueue)"
                "(&i1.Waiting_Queue))->remove(&o1);};"
                "&i1.Waiting_Processes = (&i1.Waiting_Processes) -1; "
                "GLOBAL_SEMAPHORE_MANAGER->SemaphoreUpdate(&i1)"
      end pragmas
   
   ; 
   process REMOVE_BUFFERSENDQUEUE =
        ( ? APEX_Buffer_type buffer_IN;
          ! ProcessID_type process_ID;
            Message_type message;
        )
        spec (| (| buffer_IN --> {process_ID,message} |)
              | (| process_ID ^= message ^= buffer_IN |)
              |)
      pragmas 
         C_Code "if (&i1.Queuing_Discipline == FIFO) {&o1 = ((ptFIFO_SendQueue)"
                "(&i1.WaitingOnSend_Queue))->remove(&o2);} else {&o1 = ((ptPRIORITY_SendQueue)"
                "(&i1.WaitingOnSend_Queue))->remove(&o2);};"
                "&i1.WaitingOnSend_Processes = "
                "(&i1.WaitingOnSend_Processes) -1; "
                "GLOBAL_BUFFER_MANAGER->BufferUpdate(&i1)"
      end pragmas
   
   ; 
   process REMOVALL_BLACKBOARDQUEUE =
        ( ? APEX_Blackboard_type blackboard_IN;
          ! [MAX_NUMBER_OF_PROCESSES]boolean process_range;
        )
        spec (| (| blackboard_IN --> process_range |)
              | (| process_range ^= blackboard_IN |)
              |)
      pragmas 
         C_Code "for (int i = 0; i < MAX_NUMBER_OF_PROCESSES; i++)"
                "{  &o1[i] = &i1.Waiting_Queue[i];  &i1.Waiting_Queue[i] = FALSE;}"
      end pragmas
   
   ; 
   process REMOVALL_EVENTQUEUE =
        ( ? APEX_Event_type event_IN;
          ! [MAX_NUMBER_OF_PROCESSES]boolean process_range;
        )
        spec (| (| event_IN --> process_range |)
              | (| process_range ^= event_IN |)
              |)
      pragmas 
         C_Code "for (int i = 0; i < MAX_NUMBER_OF_PROCESSES; i++)"
                "{  &o1[i] = &i1.Waiting_Queue[i];  &i1.Waiting_Queue[i] = FALSE;}"
      end pragmas
   
   ; 
   process INSERT_QUEUINGPORTSENDQUEUE =
        ( ? ProcessID_type process_ID;
            Message_type message;
            APEX_QueuingPort_type queuingPort_IN;
        )
        spec (| process_ID ^= message ^= queuingPort_IN |)
      pragmas 
         C_Code "if (&i3.Queuing_Discipline == FIFO) {((ptFIFO_SendQueue)"
                "(&i3.WaitingOnSend_Queue))->insert(&i1, &i2);} else {((ptPRIORITY_SendQueue)"
                "(&i3.WaitingOnSend_Queue))->insert(&i1, &i2);};"
                "&i3.WaitingOnSend_Processes = "
                "(&i3.WaitingOnSend_Processes) +1; "
                "GLOBAL_BUFFER_MANAGER->QueuingPortUpdate(&i3)"
      end pragmas
   
   ; 
   process INSERT_QUEUINGPORTRECEIVEQUEUE =
        ( ? ProcessID_type process_ID;
            APEX_QueuingPort_type queuingPort_IN;
        )
        spec (| process_ID ^= queuingPort_IN |)
      pragmas 
         C_Code "if (&i2.Queuing_Discipline == FIFO) {((ptFIFO_ReceiveQueue)"
                "(&i2.WaitingOnReceive_Queue))->insert(&i1);} else {((ptPRIORITY_ReceiveQueue)"
                "(&i2.WaitingOnReceive_Queue))->insert(&i1);};"
                "&i2.WaitingOnReceive_Processes = "
                "(&i2.WaitingOnReceive_Processes) +1; "
                "GLOBAL_BUFFER_MANAGER->QueuingPortUpdate(&i2)"
      end pragmas
   
   ; 
   process REMOVE_QUEUINGPORTSENDQUEUE =
        ( ? APEX_QueuingPort_type queuingPort_IN;
          ! ProcessID_type process_ID;
            Message_type message;
        )
        spec (| (| queuingPort_IN --> {process_ID,message} |)
              | (| process_ID ^= message ^= queuingPort_IN |)
              |)
      pragmas 
         C_Code "if (&i1.Queuing_Discipline == FIFO) {&o1 = ((ptFIFO_SendQueue)"
                "(&i1.WaitingOnSend_Queue))->remove(&o2);} else {&o1 = ((ptPRIORITY_SendQueue)"
                "(&i1.WaitingOnSend_Queue))->remove(&o2);};"
                "&i1.WaitingOnSend_Processes = "
                "(&i1.WaitingOnSend_Processes) -1; "
                "GLOBAL_BUFFER_MANAGER->QueuingPortUpdate(&i1)"
      end pragmas
   
   ; 
   process REMOVE_QUEUINGPORTRECEIVEQUEUE =
        ( ? APEX_QueuingPort_type queuingPort_IN;
          ! ProcessID_type process_ID;
        )
        spec (| (| queuingPort_IN --> process_ID |)
              | (| process_ID ^= queuingPort_IN |)
              |)
      pragmas 
         C_Code "if (&i1.Queuing_Discipline == FIFO) {((ptFIFO_ReceiveQueue)"
                "(&i1.WaitingOnReceive_Queue))->remove(&o1);} else {((ptPRIORITY_ReceiveQueue)"
                "(&i1.WaitingOnReceive_Queue))->remove(&o1);};"
                "&i1.WaitingOnReceive_Processes = "
                "(&i1.WaitingOnReceive_Processes) -1; "
                "GLOBAL_BUFFER_MANAGER->QueuingPortUpdate(&i1)"
      end pragmas
   
   ; 
end %PRIORITY_WaitingQueue_lib%; 
