module Process_Descriptor_low_level =
   process PROCESS_CHECKNAME =
        ( ? ProcessName_type process_name;
          ! boolean present;
            ProcessID_type process_ID;
        )
        spec (| (| process_name --> present
                 | {process_name --> process_ID} when present
                 |)
              | (| process_name ^= present
                 | process_ID ^= when present
                 |)
              |)
      pragmas 
         C_Code "&o1 = GLOBAL_PROCESS_MANAGER->ProcessCheckName(&i1, &o2)"
      end pragmas
   
   ; 
   process PROCESS_CHECKAPERIODIC =
        ( ? ProcessID_type process_ID;
          ! boolean aperiodic;
        )
      (| period := PROCESS_GETPERIOD{}(process_ID)
       | aperiodic := period=APERIODIC
       |)
      where 
      SystemTime_type period;
      end
   ; 
   process PROCESS_CHECKERRORHANDLER =
        ( ? ProcessID_type process_ID;
          ! boolean err_handler;
        )
      (| err_handler := process_ID=ERROR_HANDLER |)
   ; 
   process PROCESS_CHECKOPERATINGMODE =
        ( ! OperatingMode_type op_mode;
        )
      pragmas 
         C_Code "&o1 = GLOBAL_PROCESS_MANAGER->ProcessCheckOperatingMode()"
      end pragmas
   
   ; 
   process PROCESS_CHECKWAITINDICATOR =
        ( ? ProcessID_type process_ID;
          ! boolean waiting_ind;
        )
        spec (| (| process_ID --> waiting_ind |)
              | (| process_ID ^= waiting_ind |)
              |)
      pragmas 
         C_Code "&o1 = GLOBAL_PROCESS_MANAGER->ProcessCheckWaitIndicator(&i1)"
      end pragmas
   
   ; 
   process PROCESS_GETSTATE =
        ( ? ProcessID_type process_ID;
          ! ProcessState_type state;
        )
        spec (| (| process_ID --> state |)
              | (| process_ID ^= state |)
              |)
      pragmas 
         C_Code "&o1 = GLOBAL_PROCESS_MANAGER->ProcessGetState(&i1)"
      end pragmas
   
   ; 
   process PROCESS_GETBASICPRIORITY =
        ( ? ProcessID_type process_ID;
          ! Priority_type basic_priority;
        )
        spec (| (| process_ID --> basic_priority |)
              | (| process_ID ^= basic_priority |)
              |)
      pragmas 
         C_Code "&o1 = GLOBAL_PROCESS_MANAGER->ProcessGetBasicPriority(&i1)"
      end pragmas
   
   ; 
   process PROCESS_GETTIMECAPACITY =
        ( ? ProcessID_type process_ID;
          ! SystemTime_type time_capacity;
        )
        spec (| (| process_ID --> time_capacity |)
              | (| process_ID ^= time_capacity |)
              |)
      pragmas 
         C_Code "&o1 = GLOBAL_PROCESS_MANAGER->ProcessGetTimeCapacity(&i1)"
      end pragmas
   
   ; 
   process PROCESS_LOCKLEVELINC =
        ( ? event inc_req;
        )
      pragmas 
         C_Code " GLOBAL_PROCESS_MANAGER->ProcessIncrementLockLevel(&i1)"
      end pragmas
   
   ; 
   process PROCESS_RELEASERESOURCES =
        ( ? ProcessID_type process_ID;
        )
      pragmas 
         C_Code " "
      end pragmas
   
   ; 
   process PROCESS_RESETCONTEXT =
        ( ? ProcessID_type process_ID;
        )
      pragmas 
         C_Code 
      end pragmas
   
   ; 
   process PROCESS_RESETERRORSTATUS =
        ( ? event reset_req;
        )
      pragmas 
         C_Code " "
      end pragmas
   
   ; 
   process PROCESS_RESETLOCKLEVEL =
        ( ? event reset_req;
        )
      pragmas 
         C_Code " GLOBAL_PROCESS_MANAGER->ProcessResetLockLevel(&i1)"
      end pragmas
   
   ; 
   process PROCESS_SETALLSTATE =
        ( ? [MAX_NUMBER_OF_PROCESSES]boolean range;
            ProcessState_type state;
        )
        spec (| range ^= state |)
      pragmas 
         C_Code "GLOBAL_PROCESS_MANAGER->ProcessSetState_many(&i1, &i2)"
      end pragmas
   
   ; 
   process PROCESS_SETATTRIBUTES =
        ( ? ProcessID_type process_ID;
            ProcessAttributes_type attributes;
        )
        spec (| process_ID ^= attributes |)
      pragmas 
         C_Code "GLOBAL_PROCESS_MANAGER->ProcessSetAttributes(&i1,&i2)"
      end pragmas
   
   ; 
   process PROCESS_SETDEADLINETIME =
        ( ? ProcessID_type process_ID;
            SystemTime_type deadline;
        )
        spec (| process_ID ^= deadline |)
      pragmas 
         C_Code "GLOBAL_PROCESS_MANAGER->ProcessSetDeadlineTime(&i1, &i2)"
      end pragmas
   
   ; 
   process PROCESS_SETPRIORITY =
        ( ? ProcessID_type process_ID;
            Priority_type priority;
        )
        spec (| process_ID ^= priority |)
      pragmas 
         C_Code "GLOBAL_PROCESS_MANAGER->ProcessSetPriority(&i1, &i2)"
      end pragmas
   
   ; 
   process PROCESS_SETSTATE =
        ( ? ProcessID_type process_ID;
            ProcessState_type state;
        )
        spec (| process_ID ^= state |)
      pragmas 
         C_Code "GLOBAL_PROCESS_MANAGER->ProcessSetState(&i1, &i2)"
      end pragmas
   
   ; 
   process PROCESS_MANAGERCREATE =
        ( )
      pragmas 
         C_Code "GLOBAL_PROCESS_MANAGER = new Scheduler()"
      end pragmas
   
   ; 
   process PROCESS_COPYMESSAGE =
        ( ? Message_type message;
            ProcessID_type process_ID;
        )
        spec (| message ^= process_ID |)
      pragmas 
         C_Code "GLOBAL_PROCESS_MANAGER->ProcessCopyMessage(&i1, &i2)"
      end pragmas
   
   ; 
   process PROCESS_CHECKID =
        ( ? ProcessID_type process_ID;
          ! boolean present;
            ProcessStatus_type status;
        )
        spec (| (| process_ID --> present
                 | {process_ID --> status} when present
                 |)
              | (| process_ID ^= present
                 | status ^= when present
                 |)
              |)
      pragmas 
         C_Code "&o1 = GLOBAL_PROCESS_MANAGER->ProcessCheckID(&i1, &o2)"
      end pragmas
   
   ; 
   process PROCESS_RESETSTACK =
        ( ? ProcessID_type process_ID;
        )
      pragmas 
         C_Code " "
      end pragmas
   
   ; 
   process PROCESS_REMOVEWAITINGQUEUE =
        ( ? ProcessID_type process_ID;
        )
      pragmas 
         C_Code " "
      end pragmas
   
   ; 
   process PROCESS_PREVIOUSPREEMPTED =
        ( ? ProcessID_type process_ID;
          ! boolean preempted;
        )
        spec (| (| process_ID --> preempted |)
              | (| process_ID ^= preempted |)
              |)
      pragmas 
         C_Code "&o1 = GLOBAL_PROCESS_MANAGER->ProcessCheckPreviousPreempted(&i1)"
      end pragmas
   
   ; 
   process PROCESS_CHECKPREEMPTION =
        ( ? event req;
          ! boolean enabled;
        )
        spec (| (| req --> enabled |)
              | (| req ^= enabled |)
              |)
      pragmas 
         C_Code "&o1 = GLOBAL_PROCESS_MANAGER->ProcessCheckPreemptionStatus(&i1)"
      end pragmas
   
   ; 
   process PROCESS_SETCREATED =
        ( ? ProcessID_type process_ID;
        )
      pragmas 
         C_Code "GLOBAL_PROCESS_MANAGER->ProcessSetCreated(&i1)"
      end pragmas
   
   ; 
   process PROCESS_CHECKCREATED =
        ( ? ProcessName_type process_name;
          ! boolean created;
        )
        spec (| (| process_name --> created |)
              | (| process_name ^= created |)
              |)
      pragmas 
         C_Code "&o1 = GLOBAL_PROCESS_MANAGER->ProcessCheckCreated(&i1)"
      end pragmas
   
   ; 
   process PROCESS_GETPERIOD =
        ( ? ProcessID_type process_ID;
          ! SystemTime_type period;
        )
        spec (| (| process_ID --> period |)
              | (| process_ID ^= period |)
              |)
      pragmas 
         C_Code "&o1 = GLOBAL_PROCESS_MANAGER->ProcessGetPeriod(&i1)"
      end pragmas
   
   ; 
   process PROCESS_STARTCOUNTER =
        ( ? ProcessID_type process_ID;
            SystemTime_type delay;
        )
        spec (| process_ID ^= delay |)
      pragmas 
         C_Code "GLOBAL_PROCESS_MANAGER->ProcessStartCounter(&i1, &i2)"
      end pragmas
   
   ; 
   process PROCESS_STOPCOUNTER =
        ( ? ProcessID_type process_ID;
        )
      pragmas 
         C_Code "GLOBAL_PROCESS_MANAGER->ProcessStopCounter(&i1)"
      end pragmas
   
   ; 
   process PROCESS_GETTIME =
        ( ! SystemTime_type system_time;
        )
      pragmas 
         C_Code "&o1 = GLOBAL_PROCESS_MANAGER->ProcessGetTime()"
      end pragmas
   
   ; 
   process PROCESS_STOPALLCOUNTERS =
        ( ? [MAX_NUMBER_OF_PROCESSES]boolean range;
        )
      pragmas 
         C_Code "GLOBAL_PROCESS_MANAGER->ProcessStopCounters_many(&i1)"
      end pragmas
   
   ; 
   process PROCESS_RETRIEVEMESSAGE =
        ( ? ProcessID_type process_ID;
          ! Message_type message;
        )
        spec (| (| process_ID --> message |)
              | (| process_ID ^= message |)
              |)
      pragmas 
         C_Code "&o1 = GLOBAL_PROCESS_MANAGER->ProcessRetrieveMessage(&i1)"
      end pragmas
   
   ; 
   process PROCESS_CHECKSUSPENSIONINDICATOR =
        ( ? ProcessID_type process_ID;
          ! boolean suspended;
        )
        spec (| (| process_ID ^= suspended |)
              | (| process_ID --> suspended |)
              |)
      pragmas 
         C_Code "&o1 = GLOBAL_PROCESS_MANAGER->ProcessCheckSuspensionIndicator(&i1)"
      end pragmas
   
   ; 
   process PROCESS_SETSUSPENSIONINDICATOR =
        ( ? ProcessID_type process_ID;
            boolean suspend;
        )
        spec (| process_ID ^= suspend |)
      pragmas 
         C_Code "GLOBAL_PROCESS_MANAGER->ProcessSetSuspensionIndicator(&i1, &i2)"
      end pragmas
   
   ; 
   process PROCESS_RECORD =
        ( ? ProcessName_type name;
          ! ProcessID_type process_ID;
        )
        spec (| (| name --> process_ID |)
              | (| name ^= process_ID |)
              |)
      pragmas 
         C_Code "&o1 = GLOBAL_PROCESS_MANAGER->ProcessRecord(&i1)"
      end pragmas
   
   ; 
   process START_COUNTER =
        ( ? ProcessID_type process_ID;
            SystemTime_type timeout;
        )
      (| prag_STARTCOUNTER{}(process_ID,timeout)
       | process_ID ^= timeout
       |)
      where 
      process prag_STARTCOUNTER =
           ( ? ProcessID_type process_ID;
               SystemTime_type timeout;
           )
           spec (| process_ID ^= timeout |)
         pragmas 
            C_Code "GLOBAL_PROCESS_MANAGER->ProcessStartCounter(&i1, &i2)"
         end pragmas
      
      ; 
      end
   ; 
   process STOP_COUNTER =
        ( ? ProcessID_type process_ID;
        )
%       pragmas C_Code ""      end pragmas%
     (| prag_STOPCOUNTER{}(process_ID) |)
      where 
      process prag_STOPCOUNTER =
           ( ? ProcessID_type process_ID;
           )
         pragmas 
            C_Code "GLOBAL_PROCESS_MANAGER->ProcessStopCounter(&i1)"
         end pragmas
      
      ; 
      end
   ; 
   process RESET_RUNTIME =
        ( ? ProcessID_type process_ID;
        )
      (| prag_RESETRUNTIME{}(process_ID) |)
      where 
      process prag_RESETRUNTIME =
           ( ? ProcessID_type process_ID;
           )
         pragmas 
            C_Code "GLOBAL_PROCESS_MANAGER->ProcessResetRuntime(&i1)"
         end pragmas
      
      ; 
      end
   ; 
   process STOPALL_COUNTER =
        ( ? [MAX_NUMBER_OF_PROCESSES]boolean process_range;
        )
      (| prag_STOPALLCOUNTERS(process_range) |)
      where 
      process prag_STOPALLCOUNTERS =
           ( ? [MAX_NUMBER_OF_PROCESSES]boolean process_range;
           )
         pragmas 
            C_Code "GLOBAL_PROCESS_MANAGER->ProcessStopCounters_many(&i1)"
         end pragmas
      
      ; 
      end
   ; 
end %Process_Descriptor_low_level%; 
